@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "manhattanstyle.h"
 
@@ -34,16 +12,21 @@
 #include <utils/fancymainwindow.h>
 #include <utils/theme/theme.h>
 #include <utils/utilsicons.h>
+#include <utils/qtcassert.h>
 
 #include <QApplication>
+#include <QCheckBox>
 #include <QComboBox>
 #include <QDockWidget>
 #include <QFormLayout>
+#include <QGroupBox>
 #include <QLabel>
 #include <QLineEdit>
 #include <QMenuBar>
 #include <QPainter>
+#include <QPainterPath>
 #include <QPixmap>
+#include <QSpinBox>
 #include <QStatusBar>
 #include <QStyleFactory>
 #include <QStyleOption>
@@ -68,14 +51,23 @@ bool styleEnabled(const QWidget *widget)
     return true;
 }
 
+static bool isInUnstyledDialogOrPopup(const QWidget *widget)
+{
+    // Do not style contents of dialogs or popups without "panelwidget" property
+    const QWidget *window = widget->window();
+    if (window->property("panelwidget").toBool())
+        return false;
+    const Qt::WindowType windowType = window->windowType();
+    return (windowType == Qt::Dialog || windowType == Qt::Popup);
+}
+
 // Consider making this a QStyle state
 bool panelWidget(const QWidget *widget)
 {
     if (!widget)
         return false;
 
-    // Do not style dialogs or explicitly ignored widgets
-    if ((widget->window()->windowFlags() & Qt::WindowType_Mask) == Qt::Dialog)
+    if (isInUnstyledDialogOrPopup(widget))
         return false;
 
     if (qobject_cast<const FancyMainWindow *>(widget))
@@ -102,8 +94,7 @@ bool lightColored(const QWidget *widget)
     if (!widget)
         return false;
 
-    // Don't style dialogs or explicitly ignored widgets
-    if ((widget->window()->windowFlags() & Qt::WindowType_Mask) == Qt::Dialog)
+    if (isInUnstyledDialogOrPopup(widget))
         return false;
 
     const QWidget *p = widget;
@@ -115,6 +106,12 @@ bool lightColored(const QWidget *widget)
     return false;
 }
 
+static bool isDarkFusionStyle(const QStyle *style)
+{
+    return creatorTheme()->flag(Theme::DarkUserInterface)
+            && strcmp(style->metaObject()->className(), "QFusionStyle") == 0;
+}
+
 class ManhattanStylePrivate
 {
 public:
@@ -122,13 +119,13 @@ public:
     void init();
 
 public:
-    const QPixmap extButtonPixmap;
+    const QIcon extButtonIcon;
     const QPixmap closeButtonPixmap;
     StyleAnimator animator;
 };
 
 ManhattanStylePrivate::ManhattanStylePrivate() :
-    extButtonPixmap(Utils::Icons::TOOLBAR_EXTENSION.pixmap()),
+    extButtonIcon(Utils::Icons::TOOLBAR_EXTENSION.icon()),
     closeButtonPixmap(Utils::Icons::CLOSE_FOREGROUND.pixmap())
 {
 }
@@ -170,6 +167,14 @@ QRect ManhattanStyle::subElementRect(SubElement element, const QStyleOption *opt
 QRect ManhattanStyle::subControlRect(ComplexControl control, const QStyleOptionComplex *option,
                                      SubControl subControl, const QWidget *widget) const
 {
+#if QT_VERSION < QT_VERSION_CHECK(6, 2, 5)
+    // Workaround for QTBUG-101581, can be removed when building with Qt 6.2.5 or higher
+    if (control == CC_ScrollBar) {
+        const auto scrollbar = qstyleoption_cast<const QStyleOptionSlider *>(option);
+        if (scrollbar && qint64(scrollbar->maximum) - scrollbar->minimum > INT_MAX)
+            return QRect(); // breaks the scrollbar, but avoids the crash
+    }
+#endif
     return QProxyStyle::subControlRect(control, option, subControl, widget);
 }
 
@@ -246,14 +251,12 @@ QPalette panelPalette(const QPalette &oldPalette, bool lightColored = false)
     QPalette pal = oldPalette;
     pal.setBrush(QPalette::All, QPalette::WindowText, color);
     pal.setBrush(QPalette::All, QPalette::ButtonText, color);
-    pal.setBrush(QPalette::All, QPalette::Foreground, color);
     if (lightColored)
         color.setAlpha(100);
     else
         color = creatorTheme()->color(Theme::IconsDisabledColor);
     pal.setBrush(QPalette::Disabled, QPalette::WindowText, color);
     pal.setBrush(QPalette::Disabled, QPalette::ButtonText, color);
-    pal.setBrush(QPalette::Disabled, QPalette::Foreground, color);
     return pal;
 }
 
@@ -275,26 +278,26 @@ void ManhattanStyle::polish(QWidget *widget)
             widget->setContentsMargins(0, 0, 0, 0);
 
         widget->setAttribute(Qt::WA_LayoutUsesWidgetRect, true);
-        if (qobject_cast<QToolButton*>(widget)) {
+        // So that text isn't cutoff in line-edits, comboboxes... etc.
+        const int height = qMax(StyleHelper::navigationWidgetHeight(), QApplication::fontMetrics().height());
+        if (qobject_cast<QToolButton*>(widget) || qobject_cast<QLineEdit*>(widget)) {
             widget->setAttribute(Qt::WA_Hover);
-            widget->setMaximumHeight(StyleHelper::navigationWidgetHeight() - 2);
-        } else if (qobject_cast<QLineEdit*>(widget)) {
-            widget->setAttribute(Qt::WA_Hover);
-            widget->setMaximumHeight(StyleHelper::navigationWidgetHeight() - 2);
-        } else if (qobject_cast<QLabel*>(widget)) {
+            widget->setMaximumHeight(height - 2);
+        } else if (qobject_cast<QLabel*>(widget) || qobject_cast<QSpinBox*>(widget)
+                   || qobject_cast<QCheckBox*>(widget)) {
             widget->setPalette(panelPalette(widget->palette(), lightColored(widget)));
         } else if (widget->property("panelwidget_singlerow").toBool()) {
-            widget->setFixedHeight(StyleHelper::navigationWidgetHeight());
+            widget->setFixedHeight(height);
         } else if (qobject_cast<QStatusBar*>(widget)) {
-            widget->setFixedHeight(StyleHelper::navigationWidgetHeight() + 2);
+            widget->setFixedHeight(height + 2);
         } else if (qobject_cast<QComboBox*>(widget)) {
             const bool isLightColored = lightColored(widget);
             QPalette palette = panelPalette(widget->palette(), isLightColored);
             if (!isLightColored)
-                palette.setBrush(QPalette::All, QPalette::Foreground,
+                palette.setBrush(QPalette::All, QPalette::WindowText,
                                  creatorTheme()->color(Theme::ComboBoxTextColor));
             widget->setPalette(palette);
-            widget->setMaximumHeight(StyleHelper::navigationWidgetHeight() - 2);
+            widget->setMaximumHeight(height - 2);
             widget->setAttribute(Qt::WA_Hover);
         }
     }
@@ -305,12 +308,11 @@ void ManhattanStyle::unpolish(QWidget *widget)
     QProxyStyle::unpolish(widget);
     if (panelWidget(widget)) {
         widget->setAttribute(Qt::WA_LayoutUsesWidgetRect, false);
-        if (qobject_cast<QTabBar*>(widget))
-            widget->setAttribute(Qt::WA_Hover, false);
-        else if (qobject_cast<QToolBar*>(widget))
-            widget->setAttribute(Qt::WA_Hover, false);
-        else if (qobject_cast<QComboBox*>(widget))
+        if (qobject_cast<QTabBar*>(widget)
+                || qobject_cast<QToolBar*>(widget)
+                || qobject_cast<QComboBox*>(widget)) {
             widget->setAttribute(Qt::WA_Hover, false);
+        }
     }
 }
 
@@ -327,9 +329,6 @@ QPixmap ManhattanStyle::standardPixmap(StandardPixmap standardPixmap, const QSty
 
     QPixmap pixmap;
     switch (standardPixmap) {
-    case QStyle::SP_ToolBarHorizontalExtensionButton:
-        pixmap = d->extButtonPixmap;
-        break;
     case QStyle::SP_TitleBarCloseButton:
         pixmap = d->closeButtonPixmap;
         break;
@@ -342,7 +341,16 @@ QPixmap ManhattanStyle::standardPixmap(StandardPixmap standardPixmap, const QSty
 
 QIcon ManhattanStyle::standardIcon(StandardPixmap standardIcon, const QStyleOption *option, const QWidget *widget) const
 {
-    QIcon icon = QProxyStyle::standardIcon(standardIcon, option, widget);
+    QIcon icon;
+    switch (standardIcon) {
+    case QStyle::SP_ToolBarHorizontalExtensionButton:
+        icon = d->extButtonIcon;
+        break;
+    default:
+        icon = QProxyStyle::standardIcon(standardIcon, option, widget);
+        break;
+    }
+
     if (standardIcon == QStyle::SP_ComputerIcon) {
         // Ubuntu has in some versions a 16x16 icon, see QTCREATORBUG-12832
         const QList<QSize> &sizes = icon.availableSizes();
@@ -378,17 +386,168 @@ int ManhattanStyle::styleHint(StyleHint hint, const QStyleOption *option, const
         if (Utils::HostOsInfo::isMacHost())
             ret = QFormLayout::AllNonFixedFieldsGrow;
         break;
+    case QStyle::SH_Widget_Animation_Duration:
+        if (widget && widget->inherits("QTreeView"))
+            ret = 0;
+        break;
+    case QStyle::SH_ComboBox_AllowWheelScrolling:
+        // Turn this on only when simultaneously pressing Ctrl, to prevent accidental current
+        // index change, e.g. on a scroll view
+        ret = QGuiApplication::keyboardModifiers()
+              == (HostOsInfo::isMacHost() ? Qt::MetaModifier : Qt::ControlModifier);
+        break;
     default:
         break;
     }
     return ret;
 }
 
+static void drawPrimitiveTweakedForDarkTheme(QStyle::PrimitiveElement element,
+                                             const QStyleOption *option,
+                                             QPainter *painter, const QWidget *widget)
+{
+    const bool hasFocus = option->state & QStyle::State_HasFocus;
+    const bool isChecked = option->state & QStyle::State_On;
+    const bool isPartiallyChecked = option->state & QStyle::State_NoChange;
+    const bool isEnabled = option->state & QStyle::State_Enabled;
+    const bool isSunken = option->state & QStyle::State_Sunken;
+
+    const QColor frameColor = isEnabled ? option->palette.color(QPalette::Mid).darker(132)
+                                        : creatorTheme()->color(Theme::BackgroundColorDisabled);
+    const QColor indicatorColor = isEnabled ? option->palette.color(QPalette::Mid).darker(90)
+                                            : creatorTheme()->color(Theme::BackgroundColorDisabled);
+    const QColor bgColor = isSunken ? option->palette.color(QPalette::Mid).darker()
+                                    : option->palette.color(QPalette::Window);
+    const QColor hlColor = option->palette.color(QPalette::Highlight);
+
+    QPen framePen(hasFocus ? hlColor : frameColor, 1);
+    framePen.setJoinStyle(Qt::MiterJoin);
+    QPen indicatorPen(indicatorColor, 1);
+    indicatorPen.setJoinStyle(Qt::MiterJoin);
+
+    painter->save();
+    painter->setRenderHint(QPainter::Antialiasing);
+
+    switch (element) {
+    case QStyle::PE_Frame: {
+        const QRectF frameRectF = QRectF(option->rect).adjusted(0.5, 0.5, -0.5, -0.5);
+        painter->setPen(framePen);
+        painter->drawRect(frameRectF);
+        break;
+    }
+    case QStyle::PE_FrameLineEdit: {
+        const bool isComboBox = widget && widget->inherits("QComboBox");
+        const QRectF frameRectF =
+                QRectF(option->rect).adjusted(0.5, 0.5, isComboBox ? -8.5 : -0.5, -0.5);
+        painter->setPen(framePen);
+        painter->drawRect(frameRectF);
+        break;
+    }
+    case QStyle::PE_FrameGroupBox: {
+        // Snippet from QFusionStyle::drawPrimitive - BEGIN
+        static const int groupBoxTopMargin =  3;
+        int topMargin = 0;
+        auto control = dynamic_cast<const QGroupBox *>(widget);
+        if (control && !control->isCheckable() && control->title().isEmpty()) {
+            // Shrinking the topMargin if Not checkable AND title is empty
+            topMargin = groupBoxTopMargin;
+        } else {
+            const int exclusiveIndicatorHeight = widget ? widget->style()->pixelMetric(QStyle::PM_ExclusiveIndicatorHeight) : 0;
+            topMargin = qMax(exclusiveIndicatorHeight,
+                             option->fontMetrics.height()) + groupBoxTopMargin;
+        }
+        // Snippet from QFusionStyle::drawPrimitive - END
+
+        const QRectF frameRectF = QRectF(option->rect).adjusted(0.5, topMargin + 0.5, -0.5, -0.5);
+        painter->setPen(framePen);
+        if (isEnabled)
+            painter->setOpacity(0.5);
+        painter->drawRect(frameRectF);
+        break;
+    }
+    case QStyle::PE_IndicatorRadioButton: {
+        const double lineWidth = 1.666;
+        const double o = lineWidth / 2;
+        indicatorPen.setWidth(lineWidth);
+        painter->setPen(framePen);
+        if (isEnabled)
+            painter->setBrush(bgColor);
+        painter->drawRoundedRect(QRectF(option->rect).adjusted(o, o, -o, -o),
+                                 100, 100, Qt::RelativeSize);
+
+        if (isChecked) {
+            painter->setPen(Qt::NoPen);
+            painter->setBrush(indicatorColor);
+            const double o = 4.25;
+            painter->drawRoundedRect(QRectF(option->rect).adjusted(o, o, -o, -o),
+                                     100, 100, Qt::RelativeSize);
+        }
+        break;
+    }
+    case QStyle::PE_IndicatorCheckBox: {
+        const QRectF frameRectF = QRectF(option->rect).adjusted(0.5, 0.5, -0.5, -0.5);
+        painter->setPen(framePen);
+        if (isEnabled)
+            painter->setBrush(bgColor);
+        painter->drawRect(frameRectF);
+
+        if (isPartiallyChecked) {
+            QPen outline(indicatorColor, 1, Qt::SolidLine, Qt::FlatCap, Qt::MiterJoin);
+            painter->setPen(outline);
+            QColor fill(frameColor);
+            fill.setAlphaF(0.8f);
+            painter->setBrush(fill);
+            const double o = 3.5;
+            painter->drawRect(QRectF(option->rect).adjusted(o, o, -o, -o));
+        } else if (isChecked) {
+            const double o = 3;
+            const QRectF r = QRectF(option->rect).adjusted(o, o, -o, -o);
+            QPen checkMarkPen(indicatorColor, 1.75, Qt::SolidLine, Qt::FlatCap, Qt::MiterJoin);
+            painter->setPen(checkMarkPen);
+            painter->drawPolyline(QPolygonF({QPointF(r.left(), r.top() + r.height() / 2),
+                                             QPointF(r.left() + r.width() / 2.3, r.bottom()),
+                                             r.topRight()}));
+        }
+        break;
+    }
+    case QStyle::PE_IndicatorTabClose: {
+        QWindow *window = widget ? widget->window()->windowHandle() : nullptr;
+        QRect iconRect = QRect(0, 0, 16, 16);
+        iconRect.moveCenter(option->rect.center());
+        const QIcon::Mode mode = !isEnabled ? QIcon::Disabled : QIcon::Normal;
+        const static QIcon closeIcon = Utils::Icons::CLOSE_FOREGROUND.icon();
+        if (option->state & QStyle::State_MouseOver && widget)
+            widget->style()->drawPrimitive(QStyle::PE_PanelButtonCommand, option, painter, widget);
+        const int devicePixelRatio = widget ? widget->devicePixelRatio() : 1;
+        const QPixmap iconPx =
+                closeIcon.pixmap(window, iconRect.size() * devicePixelRatio, mode);
+        painter->drawPixmap(iconRect, iconPx);
+        break;
+    }
+    default:
+        QTC_ASSERT_STRING("Unhandled QStyle::PrimitiveElement case");
+        break;
+    }
+    painter->restore();
+}
+
 void ManhattanStyle::drawPrimitive(PrimitiveElement element, const QStyleOption *option,
                                    QPainter *painter, const QWidget *widget) const
 {
-    if (!panelWidget(widget)) {
-        QProxyStyle::drawPrimitive(element, option, painter, widget);
+    const bool isPanelWidget = panelWidget(widget);
+    if (!isPanelWidget) {
+        const bool tweakDarkTheme =
+                (element == PE_Frame
+                 || element == PE_FrameLineEdit
+                 || element == PE_FrameGroupBox
+                 || element == PE_IndicatorRadioButton
+                 || element == PE_IndicatorCheckBox
+                 || element == PE_IndicatorTabClose)
+                && isDarkFusionStyle(baseStyle());
+        if (tweakDarkTheme)
+            drawPrimitiveTweakedForDarkTheme(element, option, painter, widget);
+        else
+            QProxyStyle::drawPrimitive(element, option, painter, widget);
         return;
     }
 
@@ -519,9 +678,9 @@ void ManhattanStyle::drawPrimitive(PrimitiveElement element, const QStyleOption
                 }
                 if (option->state & State_HasFocus && (option->state & State_KeyboardFocusChange)) {
                     QColor highlight = option->palette.highlight().color();
-                    highlight.setAlphaF(0.4);
+                    highlight.setAlphaF(0.4f);
                     painter->setPen(QPen(highlight.lighter(), 1));
-                    highlight.setAlphaF(0.3);
+                    highlight.setAlphaF(0.3f);
                     painter->setBrush(highlight);
                     painter->setRenderHint(QPainter::Antialiasing);
                     const QRectF rect = option->rect;
@@ -587,10 +746,10 @@ void ManhattanStyle::drawPrimitive(PrimitiveElement element, const QStyleOption
 
             painter->setPen(Qt::NoPen);
             QColor dark = StyleHelper::borderColor();
-            dark.setAlphaF(0.4);
+            dark.setAlphaF(0.4f);
 
             QColor light = StyleHelper::baseColor();
-            light.setAlphaF(0.4);
+            light.setAlphaF(0.4f);
 
             painter->fillPath(path, light);
             painter->save();
@@ -717,19 +876,25 @@ void ManhattanStyle::drawControl(ControlElement element, const QStyleOption *opt
                     editRect.adjust(0, 0, -13, 0);
                 }
 
+                Qt::TextElideMode elideMode = Qt::ElideRight;
+                if (widget && widget->dynamicPropertyNames().contains("elidemode"))
+                    elideMode = widget->property("elidemode").value<Qt::TextElideMode>();
+
                 QLatin1Char asterisk('*');
                 int elideWidth = editRect.width();
 
-                bool notElideAsterisk = widget && widget->property("notelideasterisk").toBool()
+                bool notElideAsterisk = elideMode == Qt::ElideRight && widget
+                                        && widget->property("notelideasterisk").toBool()
                                         && cb->currentText.endsWith(asterisk)
-                                        && option->fontMetrics.width(cb->currentText) > elideWidth;
+                                        && option->fontMetrics.horizontalAdvance(cb->currentText)
+                                               > elideWidth;
 
                 QString text;
                 if (notElideAsterisk) {
-                    elideWidth -= option->fontMetrics.width(asterisk);
+                    elideWidth -= option->fontMetrics.horizontalAdvance(asterisk);
                     text = asterisk;
                 }
-                text.prepend(option->fontMetrics.elidedText(cb->currentText, Qt::ElideRight, elideWidth));
+                text.prepend(option->fontMetrics.elidedText(cb->currentText, elideMode, elideWidth));
 
                 if (creatorTheme()->flag(Theme::ComboBoxDrawTextShadow)
                     && (option->state & State_Enabled))
@@ -738,7 +903,7 @@ void ManhattanStyle::drawControl(ControlElement element, const QStyleOption *opt
                     painter->drawText(editRect.adjusted(1, 0, -1, 0), Qt::AlignLeft | Qt::AlignVCenter, text);
                 }
                 painter->setPen((option->state & State_Enabled)
-                                  ? option->palette.color(QPalette::Foreground)
+                                  ? option->palette.color(QPalette::WindowText)
                                   : creatorTheme()->color(Theme::IconsDisabledColor));
                 painter->drawText(editRect.adjusted(1, 0, -1, 0), Qt::AlignLeft | Qt::AlignVCenter, text);
 
@@ -752,7 +917,7 @@ void ManhattanStyle::drawControl(ControlElement element, const QStyleOption *opt
     case CE_SizeGrip: {
             painter->save();
             QColor dark = Qt::white;
-            dark.setAlphaF(0.1);
+            dark.setAlphaF(0.1f);
             int x, y, w, h;
             option->rect.getRect(&x, &y, &w, &h);
             int sw = qMin(h, w);
@@ -857,7 +1022,11 @@ void ManhattanStyle::drawControl(ControlElement element, const QStyleOption *opt
             }
         }
         break;
-
+    case CE_ToolButtonLabel:
+        // Directly use QCommonStyle to circumvent funny painting in QMacStyle
+        // which ignores the palette and adds an alpha
+        QCommonStyle::drawControl(element, option, painter, widget);
+        break;
     default:
         QProxyStyle::drawControl(element, option, painter, widget);
         break;
@@ -909,8 +1078,10 @@ void ManhattanStyle::drawComplexControl(ComplexControl control, const QStyleOpti
             QStyleOptionToolButton label = *toolbutton;
 
             label.palette = panelPalette(option->palette, lightColored(widget));
-            if (widget && widget->property("highlightWidget").toBool())
-                label.palette.setColor(QPalette::ButtonText, Qt::red);
+            if (widget && widget->property("highlightWidget").toBool()) {
+                label.palette.setColor(QPalette::ButtonText,
+                                       creatorTheme()->color(Theme::IconsWarningToolBarColor));
+            }
             int fw = pixelMetric(PM_DefaultFrameWidth, option, widget);
             label.rect = button.adjusted(fw, fw, -fw, -fw);
 
@@ -980,7 +1151,7 @@ void ManhattanStyle::drawComplexControl(ComplexControl control, const QStyleOpti
             QRect arrowRect((left + right) / 2 + (reverse ? 6 : -6), rect.center().y() - 3, 9, 9);
 
             if (!alignarrow) {
-                int labelwidth = option->fontMetrics.width(cb->currentText);
+                int labelwidth = option->fontMetrics.horizontalAdvance(cb->currentText);
                 if (reverse)
                     arrowRect.moveLeft(qMax(rect.width() - labelwidth - menuButtonWidth - 2, 4));
                 else
@@ -1014,7 +1185,7 @@ void ManhattanStyle::drawComplexControl(ComplexControl control, const QStyleOpti
     }
 }
 
-void ManhattanStyle::drawButtonSeparator(QPainter *painter, const QRect &rect, bool reverse) const
+void ManhattanStyle::drawButtonSeparator(QPainter *painter, const QRect &rect, bool reverse)
 {
     const QRectF borderRect = QRectF(rect).adjusted(0.5, 0.5, -0.5, -0.5);
     if (creatorTheme()->flag(Theme::FlatToolBars)) {
