@@ -1,52 +1,33 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
-#include "cppeditor.h"
-#include "cppeditorwidget.h"
+#include "followsymbol_switchmethoddecldef_test.h"
+
+#include "cppcodemodelsettings.h"
 #include "cppeditorplugin.h"
-#include "cppeditortestcase.h"
-
-#include <cpptools/cppcodemodelsettings.h>
-#include <cpptools/cppelementevaluator.h>
-#include <cpptools/cppfollowsymbolundercursor.h>
-#include <cpptools/cppvirtualfunctionassistprovider.h>
-#include <cpptools/cppvirtualfunctionproposalitem.h>
-#include <cpptools/cpptoolsreuse.h>
-#include <cpptools/cpptoolstestcase.h>
-#include <cpptools/cppmodelmanager.h>
+#include "cppeditorwidget.h"
+#include "cppelementevaluator.h"
+#include "cppfollowsymbolundercursor.h"
+#include "cppmodelmanager.h"
+#include "cpptoolsreuse.h"
+#include "cpptoolstestcase.h"
+#include "cppvirtualfunctionassistprovider.h"
+#include "cppvirtualfunctionproposalitem.h"
 
 #include <projectexplorer/kitmanager.h>
 #include <projectexplorer/projectexplorer.h>
 
+#include <texteditor/codeassist/assistinterface.h>
+#include <texteditor/codeassist/asyncprocessor.h>
 #include <texteditor/codeassist/genericproposalmodel.h>
 #include <texteditor/codeassist/iassistprocessor.h>
 #include <texteditor/codeassist/iassistproposal.h>
+#include <texteditor/texteditor.h>
 
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/idocument.h>
 
+#include <utils/environment.h>
 #include <utils/fileutils.h>
 
 #include <QDebug>
@@ -78,32 +59,34 @@
  */
 
 using namespace CPlusPlus;
-using namespace CppTools;
 using namespace TextEditor;
 using namespace Core;
 using namespace ProjectExplorer;
+using namespace Utils;
 
-class OverrideItem {
+class OverrideItem
+{
 public:
     OverrideItem() : line(0) {}
     OverrideItem(const QString &text, int line = 0) : text(text), line(line) {}
+
     bool isValid() { return line != 0; }
     QByteArray toByteArray() const
     {
         return "OverrideItem(" + text.toLatin1() + ", " + QByteArray::number(line) + ')';
     }
 
+    friend bool operator==(const OverrideItem &lhs, const OverrideItem &rhs)
+    {
+        return lhs.text == rhs.text && lhs.line == rhs.line;
+    }
+
     QString text;
     int line;
 };
 typedef QList<OverrideItem> OverrideItemList;
 Q_DECLARE_METATYPE(OverrideItem)
 
-inline bool operator==(const OverrideItem &lhs, const OverrideItem &rhs)
-{
-    return lhs.text == rhs.text && lhs.line == rhs.line;
-}
-
 QT_BEGIN_NAMESPACE
 namespace QTest {
 template<> char *toString(const OverrideItem &data)
@@ -121,8 +104,7 @@ QT_END_NAMESPACE
 
 typedef QByteArray _;
 
-namespace CppEditor {
-namespace Internal {
+namespace CppEditor::Internal::Tests {
 
 /// A fake virtual functions assist provider that runs processor->perform() already in configure()
 class VirtualFunctionTestAssistProvider : public VirtualFunctionAssistProvider
@@ -139,13 +121,13 @@ public:
     {
         VirtualFunctionAssistProvider::configure(params);
 
-        const QScopedPointer<IAssistProcessor> processor(createProcessor());
-        AssistInterface *assistInterface
-                = m_editorWidget->createAssistInterface(FollowSymbol, ExplicitlyInvoked);
-
-        const QScopedPointer<IAssistProposal> immediateProposal(
-            processor->immediateProposal(assistInterface));
-        const QScopedPointer<IAssistProposal> finalProposal(processor->perform(assistInterface));
+        std::unique_ptr<AssistInterface> assistInterface
+            = m_editorWidget->createAssistInterface(FollowSymbol, ExplicitlyInvoked);
+        const QScopedPointer<AsyncProcessor> processor(
+            dynamic_cast<AsyncProcessor *>(createProcessor(assistInterface.get())));
+        processor->setupAssistInterface(std::move(assistInterface));
+        const QScopedPointer<IAssistProposal> immediateProposal(processor->immediateProposal());
+        const QScopedPointer<IAssistProposal> finalProposal(processor->performAsync());
 
         VirtualFunctionAssistProvider::clearParams();
 
@@ -190,55 +172,9 @@ private:
     CppEditorWidget *m_editorWidget;
 };
 
-class TestDocument;
-typedef QSharedPointer<TestDocument> TestDocumentPtr;
-
-/**
- * Represents a test document.
- *
- * A TestDocument's source can contain special characters:
- *   - a '@' character denotes the initial text cursor position
- *   - a '$' character denotes the target text cursor position
- */
-class TestDocument : public Tests::TestDocument
-{
-public:
-    TestDocument(const QByteArray &source, const QByteArray &fileName)
-        : Tests::TestDocument(fileName, source)
-        , m_targetCursorPosition(m_source.indexOf(QLatin1Char('$')))
-    {
-        if (m_cursorPosition != -1 || m_targetCursorPosition != -1)
-            QVERIFY(m_cursorPosition != m_targetCursorPosition);
-
-        if (m_cursorPosition > m_targetCursorPosition) {
-            m_source.remove(m_cursorPosition, 1);
-            if (m_targetCursorPosition != -1) {
-                m_source.remove(m_targetCursorPosition, 1);
-                --m_cursorPosition;
-            }
-        } else {
-            m_source.remove(m_targetCursorPosition, 1);
-            if (m_cursorPosition != -1) {
-                m_source.remove(m_cursorPosition, 1);
-                --m_targetCursorPosition;
-            }
-        }
-    }
-
-    static TestDocumentPtr create(const QByteArray &source, const QByteArray &fileName)
-    {
-        return TestDocumentPtr(new TestDocument(source, fileName));
-    }
-
-    bool hasTargetCursorMarker() const { return m_targetCursorPosition != -1; }
-
-public:
-    int m_targetCursorPosition;
-};
-
 QList<TestDocumentPtr> singleDocument(const QByteArray &source)
 {
-    return QList<TestDocumentPtr>() << TestDocument::create(source, "file.cpp");
+    return QList<TestDocumentPtr>() << CppTestDocument::create(source, "file.cpp");
 }
 
 /**
@@ -246,7 +182,7 @@ QList<TestDocumentPtr> singleDocument(const QByteArray &source)
  * executing Follow Symbol Under Cursor or Switch Between Function Declaration/Definition
  * and checking the result.
  */
-class F2TestCase : public Tests::TestCase
+class F2TestCase : public CppEditor::Tests::TestCase
 {
 public:
     enum CppEditorAction {
@@ -258,11 +194,21 @@ public:
                const QList<TestDocumentPtr> &testFiles,
                OverrideItemList expectedVirtualFunctionProposal = OverrideItemList());
 
+    ~F2TestCase()
+    {
+        ClangdSettings::setUseClangd(m_prevUseClangd);
+    }
+
+    static ProjectExplorer::Kit *m_testKit;
 private:
     static TestDocumentPtr testFileWithInitialCursorMarker(const QList<TestDocumentPtr> &testFiles);
     static TestDocumentPtr testFileWithTargetCursorMarker(const QList<TestDocumentPtr> &testFiles);
+
+    const bool m_prevUseClangd;
 };
 
+ProjectExplorer::Kit *F2TestCase::m_testKit = nullptr;
+
 /// Creates a test case with multiple test files.
 /// Exactly one test document must be provided that contains '@', the initial position marker.
 /// Exactly one test document must be provided that contains '$', the target position marker.
@@ -270,10 +216,12 @@ private:
 F2TestCase::F2TestCase(CppEditorAction action,
                        const QList<TestDocumentPtr> &testFiles,
                        OverrideItemList expectedVirtualFunctionProposal)
+    : m_prevUseClangd(ClangdSettings::instance().useClangd())
 {
     QVERIFY(succeededSoFar());
 
-    setUseClangd();
+    if (m_testKit)
+        ClangdSettings::setUseClangd(true);
 
     // Check if there are initial and target position markers
     TestDocumentPtr initialTestFile = testFileWithInitialCursorMarker(testFiles);
@@ -285,35 +233,32 @@ F2TestCase::F2TestCase(CppEditorAction action,
 
     const QString curTestName = QLatin1String(QTest::currentTestFunction());
     const QString tag = QLatin1String(QTest::currentDataTag());
-    const bool useClangd = CppTools::ClangdSettings::instance().useClangd();
+    const bool useClangd = m_testKit;
     if (useClangd) {
-        if (curTestName == "test_FollowSymbolUnderCursor_QObject_connect"
-                || curTestName == "test_FollowSymbolUnderCursor_QObject_oldStyleConnect") {
+        if (curTestName == "testFollowSymbolQObjectConnect"
+                || curTestName == "testFollowSymbolQObjectOldStyleConnect") {
             QSKIP("TODO: Implement fall-back");
         }
-        if (curTestName == "test_FollowSymbolUnderCursor_classOperator" && tag == "backward")
+        if (curTestName == "testFollowClassOperator" && tag == "backward")
             QSKIP("clangd goes to operator name first");
         if (tag.toLower().contains("fuzzy"))
             QSKIP("fuzzy matching is not supposed to work with clangd"); // TODO: Implement fallback as we do with libclang
         if (tag == "baseClassFunctionIntroducedByUsingDeclaration")
             QSKIP("clangd points to the using declaration");
-        if (tag == "classDestructor" || tag == "fromDestructorDefinitionSymbol"
-                || tag == "fromDestructorBody") {
-            QSKIP("clangd wants the cursor before the ~ character");
-        }
-        if (curTestName == "test_FollowSymbolUnderCursor_classOperator_inOp")
+        if (curTestName == "testFollowClassOperatorInOp")
             QSKIP("clangd goes to operator name first");
     }
 
     // Write files to disk
-    CppTools::Tests::TemporaryDir temporaryDir;
+    CppEditor::Tests::TemporaryDir temporaryDir;
     QVERIFY(temporaryDir.isValid());
     QString projectFileContent = "CppApplication { files: [";
-    foreach (TestDocumentPtr testFile, testFiles) {
+   for (TestDocumentPtr testFile : testFiles) {
         QVERIFY(testFile->baseDirectory().isEmpty());
         testFile->setBaseDirectory(temporaryDir.path());
         QVERIFY(testFile->writeToDisk());
-        projectFileContent += QString::fromLatin1("\"%1\",").arg(testFile->filePath());
+        projectFileContent += QString::fromLatin1("\"%1\",")
+                .arg(testFile->filePath().toString());
     }
     projectFileContent += "]}\n";
 
@@ -326,29 +271,28 @@ F2TestCase::F2TestCase(CppEditorAction action,
     } projectCloser;
 
     if (useClangd) {
-        TestDocument projectFile(projectFileContent.toUtf8(), "project.qbs");
+        CppTestDocument projectFile("project.qbs", projectFileContent.toUtf8());
         projectFile.setBaseDirectory(temporaryDir.path());
         QVERIFY(projectFile.writeToDisk());
         const auto openProjectResult = ProjectExplorerPlugin::openProject(projectFile.filePath());
         QVERIFY2(openProjectResult && openProjectResult.project(),
                  qPrintable(openProjectResult.errorMessage()));
         projectCloser.setProject(openProjectResult.project());
-        openProjectResult.project()->configureAsExampleProject(
-                    CppEditorPlugin::instance()->m_testKit);
+        openProjectResult.project()->configureAsExampleProject(m_testKit);
 
         // Wait until project is fully indexed.
-        QVERIFY(CppTools::Tests::waitForSignalOrTimeout(openProjectResult.project(),
-                &Project::indexingFinished, CppTools::Tests::clangdIndexingTimeout()));
+        QVERIFY(CppEditor::Tests::waitForSignalOrTimeout(openProjectResult.project(),
+                &Project::indexingFinished, CppEditor::Tests::clangdIndexingTimeout()));
     }
 
     // Update Code Model
-    QSet<QString> filePaths;
-    foreach (const TestDocumentPtr &testFile, testFiles)
+    QSet<FilePath> filePaths;
+   for (const TestDocumentPtr &testFile : testFiles)
         filePaths << testFile->filePath();
     QVERIFY(parseFiles(filePaths));
 
     // Open Files
-    foreach (TestDocumentPtr testFile, testFiles) {
+   for (TestDocumentPtr testFile : testFiles) {
         QVERIFY(openCppEditor(testFile->filePath(), &testFile->m_editor,
                               &testFile->m_editorWidget));
         if (!useClangd) // Editors get closed when unloading project.
@@ -387,16 +331,15 @@ F2TestCase::F2TestCase(CppEditorAction action,
     switch (action) {
     case FollowSymbolUnderCursorAction: {
         CppEditorWidget *widget = initialTestFile->m_editorWidget;
-        FollowSymbolInterface &delegate = CppModelManager::instance()->followSymbolInterface();
-        auto* builtinFollowSymbol = dynamic_cast<FollowSymbolUnderCursor *>(&delegate);
-        if (!builtinFollowSymbol) {
-            if (curTestName == "test_FollowSymbolUnderCursor_QTCREATORBUG7903")
+        if (CppModelManager::instance()->isClangCodeModelActive()) {
+            if (curTestName == "testFollowSymbolQTCREATORBUG7903")
                 QSKIP((curTestName + " is not supported by Clang FollowSymbol").toLatin1());
-            widget->inTestMode = true;
+            widget->enableTestMode();
             widget->openLinkUnderCursor();
             break;
         }
 
+        FollowSymbolUnderCursor *builtinFollowSymbol = &CppModelManager::builtinFollowSymbol();
         QSharedPointer<VirtualFunctionAssistProvider> original
                 = builtinFollowSymbol->virtualFunctionAssistProvider();
 
@@ -429,8 +372,8 @@ F2TestCase::F2TestCase(CppEditorAction action,
         QEXPECT_FAIL("infiniteLoopLocalTypedef_QTCREATORBUG-11999",
                      "clangd bug: Go to definition does not return", Abort);
         if (expectedVirtualFunctionProposal.size() <= 1) {
-            QVERIFY(CppTools::Tests::waitForSignalOrTimeout(EditorManager::instance(),
-                                                            &EditorManager::linkOpened, 10000));
+            QVERIFY(CppEditor::Tests::waitForSignalOrTimeout(EditorManager::instance(),
+                                                             &EditorManager::linkOpened, 10000));
         } else {
             QTimer t;
             QEventLoop l;
@@ -462,7 +405,7 @@ F2TestCase::F2TestCase(CppEditorAction action,
     BaseTextEditor *currentTextEditor = dynamic_cast<BaseTextEditor*>(currentEditor);
     QVERIFY(currentTextEditor);
 
-    QCOMPARE(currentTextEditor->document()->filePath().toString(), targetTestFile->filePath());
+    QCOMPARE(currentTextEditor->document()->filePath(), targetTestFile->filePath());
     int expectedLine, expectedColumn;
     if (useClangd && expectedVirtualFunctionProposal.size() == 1) {
         expectedLine = expectedVirtualFunctionProposal.first().line;
@@ -471,11 +414,15 @@ F2TestCase::F2TestCase(CppEditorAction action,
     } else {
         currentTextEditor->convertPosition(targetTestFile->m_targetCursorPosition,
                                            &expectedLine, &expectedColumn);
+        if (useClangd && (tag == "classDestructor" || tag == "fromDestructorDefinitionSymbol"
+                || tag == "fromDestructorBody")) {
+            --expectedColumn; // clangd goes before the ~, built-in code model after
+        }
     }
 //    qDebug() << "Expected line:" << expectedLine;
 //    qDebug() << "Expected column:" << expectedColumn;
 
-    if (!CppTools::ClangdSettings::instance().useClangd()) {
+    if (!useClangd) {
         QEXPECT_FAIL("globalVarFromEnum", "Contributor works on a fix.", Abort);
         QEXPECT_FAIL("matchFunctionSignature_Follow_5", "foo(int) resolved as CallAST", Abort);
     }
@@ -515,7 +462,7 @@ F2TestCase::F2TestCase(CppEditorAction action,
 
 TestDocumentPtr F2TestCase::testFileWithInitialCursorMarker(const QList<TestDocumentPtr> &testFiles)
 {
-    foreach (const TestDocumentPtr testFile, testFiles) {
+   for (const TestDocumentPtr &testFile : testFiles) {
         if (testFile->hasCursorMarker())
             return testFile;
     }
@@ -524,40 +471,38 @@ TestDocumentPtr F2TestCase::testFileWithInitialCursorMarker(const QList<TestDocu
 
 TestDocumentPtr F2TestCase::testFileWithTargetCursorMarker(const QList<TestDocumentPtr> &testFiles)
 {
-    foreach (const TestDocumentPtr testFile, testFiles) {
+   for (const TestDocumentPtr &testFile : testFiles) {
         if (testFile->hasTargetCursorMarker())
             return testFile;
     }
     return TestDocumentPtr();
 }
 
-} // namespace Internal
-} // namespace CppEditor
+} // namespace CppEditor::Internal::Tests
 
-Q_DECLARE_METATYPE(QList<CppEditor::Internal::TestDocumentPtr>)
+Q_DECLARE_METATYPE(QList<CppEditor::Internal::Tests::TestDocumentPtr>)
 
-namespace CppEditor {
-namespace Internal {
+namespace CppEditor::Internal::Tests {
 
-void CppEditorPlugin::initTestCase()
+void FollowSymbolTest::initTestCase()
 {
-    const QString clangdFromEnv = qEnvironmentVariable("QTC_CLANGD");
+    const QString clangdFromEnv = Utils::qtcEnvironmentVariable("QTC_CLANGD");
     if (clangdFromEnv.isEmpty())
         return;
-    ClangdSettings::setClangdFilePath(Utils::FilePath::fromString(clangdFromEnv));
+    ClangdSettings::setClangdFilePath(Utils::FilePath::fromUserInput(clangdFromEnv));
     const auto clangd = ClangdSettings::instance().clangdFilePath();
     if (clangd.isEmpty() || !clangd.exists())
         return;
 
     // Find suitable kit.
-    m_testKit = Utils::findOr(KitManager::kits(), nullptr, [](const Kit *k) {
+    F2TestCase::m_testKit = Utils::findOr(KitManager::kits(), nullptr, [](const Kit *k) {
         return k->isValid();
     });
-    if (!m_testKit)
+    if (!F2TestCase::m_testKit)
         QSKIP("This test requires at least one kit to be present");
 }
 
-void CppEditorPlugin::test_SwitchMethodDeclarationDefinition_data()
+void FollowSymbolTest::testSwitchMethodDeclDef_data()
 {
     QTest::addColumn<QByteArray>("header");
     QTest::addColumn<QByteArray>("source");
@@ -849,19 +794,19 @@ void CppEditorPlugin::test_SwitchMethodDeclarationDefinition_data()
                  "Foo::@operator int() const { return {}; }\n");
 }
 
-void CppEditorPlugin::test_SwitchMethodDeclarationDefinition()
+void FollowSymbolTest::testSwitchMethodDeclDef()
 {
     QFETCH(QByteArray, header);
     QFETCH(QByteArray, source);
 
     const QList<TestDocumentPtr> testFiles = QList<TestDocumentPtr>()
-        << TestDocument::create(header, "file.h")
-        << TestDocument::create(source, "file.cpp");
+        << CppTestDocument::create(header, "file.h")
+        << CppTestDocument::create(source, "file.cpp");
 
     F2TestCase(F2TestCase::SwitchBetweenMethodDeclarationDefinitionAction, testFiles);
 }
 
-void CppEditorPlugin::test_FollowSymbolUnderCursor_data()
+void FollowSymbolTest::testFollowSymbol_data()
 {
     QTest::addColumn<QByteArray>("source");
 
@@ -1306,13 +1251,13 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_data()
     );
 }
 
-void CppEditorPlugin::test_FollowSymbolUnderCursor()
+void FollowSymbolTest::testFollowSymbol()
 {
     QFETCH(QByteArray, source);
     F2TestCase(F2TestCase::FollowSymbolUnderCursorAction, singleDocument(source));
 }
 
-void CppEditorPlugin::test_FollowSymbolUnderCursor_QTCREATORBUG7903_data()
+void FollowSymbolTest::testFollowSymbolQTCREATORBUG7903_data()
 {
     QTest::addColumn<QByteArray>("source");
     QTest::newRow("using_QTCREATORBUG7903_globalNamespace") << _(
@@ -1351,13 +1296,13 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_QTCREATORBUG7903_data()
     );
 }
 
-void CppEditorPlugin::test_FollowSymbolUnderCursor_QTCREATORBUG7903()
+void FollowSymbolTest::testFollowSymbolQTCREATORBUG7903()
 {
     QFETCH(QByteArray, source);
     F2TestCase(F2TestCase::FollowSymbolUnderCursorAction, singleDocument(source));
 }
 
-void CppEditorPlugin::test_FollowSymbolUnderCursor_followCall_data()
+void FollowSymbolTest::testFollowCall_data()
 {
     QTest::addColumn<QByteArray>("variableDeclaration"); // without semicolon, can be ""
     QTest::addColumn<QByteArray>("callArgument");
@@ -1396,7 +1341,7 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_followCall_data()
             << _("const Bar &");
 }
 
-void CppEditorPlugin::test_FollowSymbolUnderCursor_followCall()
+void FollowSymbolTest::testFollowCall()
 {
     QFETCH(QByteArray, variableDeclaration);
     QFETCH(QByteArray, callArgument);
@@ -1424,92 +1369,92 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_followCall()
     F2TestCase(F2TestCase::FollowSymbolUnderCursorAction, singleDocument(source));
 }
 
-void CppEditorPlugin::test_FollowSymbolUnderCursor_multipleDocuments_data()
+void FollowSymbolTest::testFollowSymbolMultipleDocuments_data()
 {
     QTest::addColumn<QList<TestDocumentPtr> >("documents");
 
     QTest::newRow("skipForwardDeclarationBasic") << (QList<TestDocumentPtr>()
-        << TestDocument::create("class $Foo {};\n",
+        << CppTestDocument::create("class $Foo {};\n",
                                 "defined.h")
-        << TestDocument::create("class Foo;\n"
+        << CppTestDocument::create("class Foo;\n"
                                 "@Foo foo;\n",
                                 "forwardDeclaredAndUsed.h")
     );
 
     QTest::newRow("skipForwardDeclarationTemplates") << (QList<TestDocumentPtr>()
-        << TestDocument::create("template <class E> class $Container {};\n",
+        << CppTestDocument::create("template <class E> class $Container {};\n",
                                 "defined.h")
-        << TestDocument::create("template <class E> class Container;\n"
+        << CppTestDocument::create("template <class E> class Container;\n"
                                 "@Container<int> container;\n",
                                 "forwardDeclaredAndUsed.h")
     );
 
     QTest::newRow("matchFunctionSignature") << (QList<TestDocumentPtr>()
-        << TestDocument::create("class Foo {\n"
+        << CppTestDocument::create("class Foo {\n"
                                 "    void @foo(int);\n"
                                 "    void foo() {}\n"
                                 "};\n",
                                 "foo.h")
-        << TestDocument::create("#include \"foo.h\"\n"
+        << CppTestDocument::create("#include \"foo.h\"\n"
                                 "void Foo::$foo(int) {}\n",
                                 "foo.cpp")
     );
 
     QTest::newRow("matchFunctionSignature2") << (QList<TestDocumentPtr>()
-        << TestDocument::create("namespace N { class C; }\n"
+        << CppTestDocument::create("namespace N { class C; }\n"
                                 "bool *@fun(N::C *) const;\n",
                                 "foo.h")
-        << TestDocument::create("#include \"foo.h\"\n"
+        << CppTestDocument::create("#include \"foo.h\"\n"
                                 "using namespace N;\n"
                                 "bool *$fun(C *) const {}\n",
                                 "foo.cpp")
     );
 
     QTest::newRow("matchFunctionSignatureFuzzy1Forward") << (QList<TestDocumentPtr>()
-        << TestDocument::create("class Foo {\n"
+        << CppTestDocument::create("class Foo {\n"
                                 "    void @foo(int);\n"
                                 "    void foo();\n"
                                 "};\n",
                                 "foo.h")
-        << TestDocument::create("#include \"foo.h\"\n"
+        << CppTestDocument::create("#include \"foo.h\"\n"
                                 "void Foo::$foo() {}\n",
                                 "foo.cpp")
     );
 
     QTest::newRow("matchFunctionSignatureFuzzy1Backward") << (QList<TestDocumentPtr>()
-        << TestDocument::create("class Foo {\n"
+        << CppTestDocument::create("class Foo {\n"
                                 "    void $foo(int);\n"
                                 "};\n",
                                 "foo.h")
-        << TestDocument::create("#include \"foo.h\"\n"
+        << CppTestDocument::create("#include \"foo.h\"\n"
                                 "void Foo::@foo() {}\n",
                                 "foo.cpp")
     );
 
     QTest::newRow("matchFunctionSignatureFuzzy2Forward") << (QList<TestDocumentPtr>()
-        << TestDocument::create("class Foo {\n"
+        << CppTestDocument::create("class Foo {\n"
                                 "    void foo(int);\n"
                                 "    void @foo();\n"
                                 "};\n",
                                 "foo.h")
-        << TestDocument::create("#include \"foo.h\"\n"
+        << CppTestDocument::create("#include \"foo.h\"\n"
                                 "void Foo::$foo(int) {}\n",
                                 "foo.cpp")
     );
 
     QTest::newRow("matchFunctionSignatureFuzzy2Backward") << (QList<TestDocumentPtr>()
-        << TestDocument::create("class Foo {\n"
+        << CppTestDocument::create("class Foo {\n"
                                 "    void $foo();\n"
                                 "};\n",
                                 "foo.h")
-        << TestDocument::create("#include \"foo.h\"\n"
+        << CppTestDocument::create("#include \"foo.h\"\n"
                                 "void Foo::@foo(int) {}\n",
                                 "foo.cpp")
     );
 
     QTest::newRow("globalVar") << QList<TestDocumentPtr>{
-            TestDocument::create("namespace NS { extern int @globalVar; }\n", "file.h"),
-            TestDocument::create(
+            CppTestDocument::create("namespace NS { extern int @globalVar; }\n", "file.h"),
+            CppTestDocument::create(
                 "int globalVar;\n"
                 "namespace NS {\n"
                 "extern int globalVar;\n"
@@ -1517,7 +1462,7 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_multipleDocuments_data()
                 "}\n", "file.cpp")};
 
     QTest::newRow("staticMemberVar") << QList<TestDocumentPtr>{
-        TestDocument::create(
+        CppTestDocument::create(
             "namespace NS {\n"
             "class Test {\n"
             "    static int @var;\n"
@@ -1525,7 +1470,7 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_multipleDocuments_data()
             "class OtherClass { static int var; };\n"
             "}\n"
             "class OtherClass { static int var; };\n", "file.h"),
-        TestDocument::create(
+        CppTestDocument::create(
             "#include \"file.h\"\n"
             "int var;\n"
             "int OtherClass::var;\n"
@@ -1535,13 +1480,13 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_multipleDocuments_data()
             "}\n", "file.cpp")};
 }
 
-void CppEditorPlugin::test_FollowSymbolUnderCursor_multipleDocuments()
+void FollowSymbolTest::testFollowSymbolMultipleDocuments()
 {
     QFETCH(QList<TestDocumentPtr>, documents);
     F2TestCase(F2TestCase::FollowSymbolUnderCursorAction, documents);
 }
 
-void CppEditorPlugin::test_FollowSymbolUnderCursor_QObject_connect_data()
+void FollowSymbolTest::testFollowSymbolQObjectConnect_data()
 {
 #define TAG(str) secondQObjectParam ? str : str ", no 2nd QObject"
     QTest::addColumn<char>("start");
@@ -1593,7 +1538,7 @@ static void selectMarker(QByteArray *source, char marker, char number)
     }
 }
 
-void CppEditorPlugin::test_FollowSymbolUnderCursor_QObject_connect()
+void FollowSymbolTest::testFollowSymbolQObjectConnect()
 {
     QFETCH(char, start);
     QFETCH(char, target);
@@ -1644,7 +1589,7 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_QObject_connect()
     F2TestCase(F2TestCase::FollowSymbolUnderCursorAction, singleDocument(source));
 }
 
-void CppEditorPlugin::test_FollowSymbolUnderCursor_QObject_oldStyleConnect()
+void FollowSymbolTest::testFollowSymbolQObjectOldStyleConnect()
 {
     const QByteArray source =
             "class O : public QObject {\n"
@@ -1664,14 +1609,14 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_QObject_oldStyleConnect()
     F2TestCase(F2TestCase::FollowSymbolUnderCursorAction, singleDocument(source));
 }
 
-void CppEditorPlugin::test_FollowSymbolUnderCursor_classOperator_onOperatorToken_data()
+void FollowSymbolTest::testFollowClassOperatorOnOperatorToken_data()
 {
     QTest::addColumn<bool>("toDeclaration");
     QTest::newRow("forward") << false;
     QTest::newRow("backward") << true;
 }
 
-void CppEditorPlugin::test_FollowSymbolUnderCursor_classOperator_onOperatorToken()
+void FollowSymbolTest::testFollowClassOperatorOnOperatorToken()
 {
     QFETCH(bool, toDeclaration);
 
@@ -1687,12 +1632,12 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_classOperator_onOperatorToken
     F2TestCase(F2TestCase::FollowSymbolUnderCursorAction, singleDocument(source));
 }
 
-void CppEditorPlugin::test_FollowSymbolUnderCursor_classOperator_data()
+void FollowSymbolTest::testFollowClassOperator_data()
 {
-    test_FollowSymbolUnderCursor_classOperator_onOperatorToken_data();
+    testFollowClassOperatorOnOperatorToken_data();
 }
 
-void CppEditorPlugin::test_FollowSymbolUnderCursor_classOperator()
+void FollowSymbolTest::testFollowClassOperator()
 {
     QFETCH(bool, toDeclaration);
 
@@ -1712,12 +1657,12 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_classOperator()
     F2TestCase(F2TestCase::FollowSymbolUnderCursorAction, singleDocument(source));
 }
 
-void CppEditorPlugin::test_FollowSymbolUnderCursor_classOperator_inOp_data()
+void FollowSymbolTest::testFollowClassOperatorInOp_data()
 {
-    test_FollowSymbolUnderCursor_classOperator_onOperatorToken_data();
+    testFollowClassOperatorOnOperatorToken_data();
 }
 
-void CppEditorPlugin::test_FollowSymbolUnderCursor_classOperator_inOp()
+void FollowSymbolTest::testFollowClassOperatorInOp()
 {
     QFETCH(bool, toDeclaration);
 
@@ -1737,7 +1682,7 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_classOperator_inOp()
     F2TestCase(F2TestCase::FollowSymbolUnderCursorAction, singleDocument(source));
 }
 
-void CppEditorPlugin::test_FollowSymbolUnderCursor_virtualFunctionCall_data()
+void FollowSymbolTest::testFollowVirtualFunctionCall_data()
 {
     QTest::addColumn<QByteArray>("source");
     QTest::addColumn<OverrideItemList>("results");
@@ -2036,7 +1981,7 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_virtualFunctionCall_data()
             << OverrideItem(QLatin1String("Derived::virt"), 2));
 }
 
-void CppEditorPlugin::test_FollowSymbolUnderCursor_virtualFunctionCall()
+void FollowSymbolTest::testFollowVirtualFunctionCall()
 {
     QFETCH(QByteArray, source);
     QFETCH(OverrideItemList, results);
@@ -2045,15 +1990,15 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_virtualFunctionCall()
 }
 
 /// Check: Base classes can be found although these might be defined in distinct documents.
-void CppEditorPlugin::test_FollowSymbolUnderCursor_virtualFunctionCall_multipleDocuments()
+void FollowSymbolTest::testFollowVirtualFunctionCallMultipleDocuments()
 {
     QList<TestDocumentPtr> testFiles = QList<TestDocumentPtr>()
-            << TestDocument::create("struct A { virtual void virt(int) = 0; };\n",
+            << CppTestDocument::create("struct A { virtual void virt(int) = 0; };\n",
                                     "a.h")
-            << TestDocument::create("#include \"a.h\"\n"
+            << CppTestDocument::create("#include \"a.h\"\n"
                                     "struct B : A { void virt(int) {} };\n",
                                     "b.h")
-            << TestDocument::create("#include \"a.h\"\n"
+            << CppTestDocument::create("#include \"a.h\"\n"
                                     "void f(A *o) { o->$@virt(42); }\n",
                                     "u.cpp")
             ;
@@ -2065,8 +2010,7 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_virtualFunctionCall_multipleD
     F2TestCase(F2TestCase::FollowSymbolUnderCursorAction, testFiles, finalResults);
 }
 
-} // namespace Internal
-} // namespace CppEditor
+} // namespace CppEditor::Internal::Tests
 
 /*
 Potential test cases improving name lookup.
