@@ -1,36 +1,16 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "breakpoint.h"
 
-#include "debuggeractions.h"
-#include "debuggercore.h"
 #include "debuggerprotocol.h"
 
 #include <projectexplorer/abi.h>
+
+#include <utils/environment.h>
+#include <utils/fileutils.h>
 #include <utils/qtcassert.h>
+#include <utils/stringutils.h>
 
 #include <QDebug>
 #include <QFileInfo>
@@ -134,7 +114,7 @@ bool BreakpointParameters::conditionsMatch(const QString &other) const
 
 void BreakpointParameters::updateLocation(const QString &location)
 {
-    if (location.size()) {
+    if (!location.isEmpty()) {
         int pos = location.indexOf(':');
         lineNumber = location.mid(pos + 1).toInt();
         QString file = location.left(pos);
@@ -142,7 +122,7 @@ void BreakpointParameters::updateLocation(const QString &location)
             file = file.mid(1, file.size() - 2);
         QFileInfo fi(file);
         if (fi.isReadable())
-            fileName = fi.absoluteFilePath();
+            fileName = Utils::FilePath::fromFileInfo(fi);
     }
 }
 
@@ -151,13 +131,14 @@ bool BreakpointParameters::isQmlFileAndLineBreakpoint() const
     if (type != BreakpointByFileAndLine)
         return false;
 
-    QString qmlExtensionString = QString::fromLocal8Bit(qgetenv("QTC_QMLDEBUGGER_FILEEXTENSIONS"));
+    QString qmlExtensionString = Utils::qtcEnvironmentVariable("QTC_QMLDEBUGGER_FILEEXTENSIONS");
     if (qmlExtensionString.isEmpty())
-        qmlExtensionString = ".qml;.js";
+        qmlExtensionString = ".qml;.js;.mjs";
 
-    const auto qmlFileExtensions = qmlExtensionString.splitRef(';', QString::SkipEmptyParts);
-    for (QStringRef extension : qmlFileExtensions) {
-        if (fileName.endsWith(extension, Qt::CaseInsensitive))
+    const auto qmlFileExtensions = qmlExtensionString.split(';', Qt::SkipEmptyParts);
+    const QString file = fileName.toString();
+    for (const QString &extension : qmlFileExtensions) {
+        if (file.endsWith(extension, Qt::CaseInsensitive))
             return true;
     }
     return false;
@@ -291,7 +272,6 @@ void BreakpointParameters::updateFromGdbOutput(const GdbMi &bkpt)
     QString originalLocation;
     QString file;
     QString fullName;
-    QString internalId;
 
     enabled = true;
     pending = false;
@@ -346,7 +326,7 @@ void BreakpointParameters::updateFromGdbOutput(const GdbMi &bkpt)
                 QString what = bkpt["what"].data();
                 if (what.startsWith("*0x")) {
                     type = WatchpointAtAddress;
-                    address = what.mid(1).toULongLong(0, 0);
+                    address = what.mid(1).toULongLong(nullptr, 0);
                 } else {
                     type = WatchpointAtExpression;
                     expression = what;
@@ -378,7 +358,7 @@ void BreakpointParameters::updateFromGdbOutput(const GdbMi &bkpt)
     QString name;
     if (!fullName.isEmpty()) {
         name = cleanupFullName(fullName);
-        fileName = name;
+        fileName = Utils::FilePath::fromString(name);
         //if (data->markerFileName().isEmpty())
         //    data->setMarkerFileName(name);
     } else {
@@ -387,7 +367,7 @@ void BreakpointParameters::updateFromGdbOutput(const GdbMi &bkpt)
         // gdb's own. No point in assigning markerFileName for now.
     }
     if (!name.isEmpty())
-        fileName = name;
+        fileName = Utils::FilePath::fromString(name);
 
     if (fileName.isEmpty())
         updateLocation(originalLocation);
