@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "modeltotextmerger.h"
 #include "modelnodepositionrecalculator.h"
@@ -73,20 +51,26 @@ void ModelToTextMerger::nodeRemoved(const ModelNode &removedNode, const NodeAbst
 
 void ModelToTextMerger::propertiesRemoved(const QList<AbstractProperty>& propertyList)
 {
-    foreach (const AbstractProperty &property, propertyList) {
-        if (isInHierarchy(property) && !property.isDefaultProperty())
+    for (const AbstractProperty &property : propertyList) {
+        // Default property that has actual binding/value should be removed
+        if (isInHierarchy(property) && (!property.isDefaultProperty()
+                                        || property.isBindingProperty()
+                                        || property.isVariantProperty()
+                                        || property.isNodeProperty())) {
             schedule(new RemovePropertyRewriteAction(property));
+        }
     }
 }
 
 void ModelToTextMerger::propertiesChanged(const QList<AbstractProperty>& propertyList, PropertyChangeFlags propertyChange)
 {
-    foreach (const AbstractProperty &property, propertyList) {
+    const TextEditor::TabSettings tabSettings = m_rewriterView->textModifier()->tabSettings();
+    for (const AbstractProperty &property : propertyList) {
 
         ModelNode containedModelNode;
-        const int indentDepth = m_rewriterView->textModifier()->indentDepth();
         const QString propertyTextValue = QmlTextGenerator(propertyOrder(),
-                                                           indentDepth)(property);
+                                                           tabSettings,
+                                                           tabSettings.m_indentSize)(property);
 
         switch (propertyChange) {
         case AbstractView::PropertiesAdded:
@@ -162,14 +146,18 @@ void ModelToTextMerger::nodeReparented(const ModelNode &node, const NodeAbstract
         switch (propertyChange) {
         case AbstractView::PropertiesAdded:
             schedule(new AddPropertyRewriteAction(newPropertyParent,
-                                                  QmlTextGenerator(propertyOrder())(node),
+                                                  QmlTextGenerator(propertyOrder(),
+                                                                   m_rewriterView->textModifier()
+                                                                       ->tabSettings())(node),
                                                   propertyType(newPropertyParent),
                                                   node));
             break;
 
         case AbstractView::NoAdditionalChanges:
             schedule(new ChangePropertyRewriteAction(newPropertyParent,
-                                                     QmlTextGenerator(propertyOrder())(node),
+                                                     QmlTextGenerator(propertyOrder(),
+                                                                      m_rewriterView->textModifier()
+                                                                          ->tabSettings())(node),
                                                      propertyType(newPropertyParent),
                                                      node));
             break;
@@ -212,8 +200,8 @@ void ModelToTextMerger::applyChanges()
         return;
 
     dumpRewriteActions(QStringLiteral("Before compression"));
-    RewriteActionCompressor compress(propertyOrder());
-    compress(m_rewriteActions);
+    RewriteActionCompressor compress(propertyOrder(), m_rewriterView->positionStorage());
+    compress(m_rewriteActions, m_rewriterView->textModifier()->tabSettings());
     dumpRewriteActions(QStringLiteral("After compression"));
 
     if (m_rewriteActions.isEmpty())
@@ -221,7 +209,8 @@ void ModelToTextMerger::applyChanges()
 
     m_rewriterView->emitCustomNotification(StartRewriterApply);
 
-    Document::MutablePtr tmpDocument(Document::create(QStringLiteral("<ModelToTextMerger>"), Dialect::Qml));
+    Document::MutablePtr tmpDocument(
+        Document::create(Utils::FilePath::fromString("<ModelToTextMerger>"), Dialect::Qml));
     tmpDocument->setSource(m_rewriterView->textModifier()->text());
     if (!tmpDocument->parseQml()) {
         qDebug() << "*** Possible problem: QML file wasn't parsed correctly.";
@@ -248,7 +237,7 @@ void ModelToTextMerger::applyChanges()
         textModifier->deactivateChangeSignals();
         textModifier->startGroup();
 
-        for (auto action : m_rewriteActions) {
+        for (auto action : std::as_const(m_rewriteActions)) {
             if (DebugRewriteActions)
                 qDebug() << "Next rewrite action:" << qPrintable(action->info());
 
@@ -310,7 +299,7 @@ void ModelToTextMerger::reindent(const QMap<int, int> &dirtyAreas) const
     Utils::sort(offsets);
     TextModifier *textModifier = m_rewriterView->textModifier();
 
-    foreach (const int offset, offsets) {
+    for (const int offset : std::as_const(offsets)) {
         const int length = dirtyAreas[offset];
         textModifier->indent(offset, length);
     }
@@ -346,6 +335,8 @@ QmlRefactoring::PropertyType ModelToTextMerger::propertyType(const AbstractPrope
         return QmlRefactoring::ObjectBinding;
     else if (property.isVariantProperty())
         return QmlRefactoring::ScriptBinding;
+    else if (property.isSignalDeclarationProperty())
+        return QmlRefactoring::ScriptBinding;
 
     Q_ASSERT(false); //Cannot convert property type
     return QmlRefactoring::Invalid;
@@ -362,12 +353,34 @@ PropertyNameList ModelToTextMerger::propertyOrder()
         PropertyName("y"),
         PropertyName("width"),
         PropertyName("height"),
+        PropertyName("opacity"),
+        PropertyName("visible"),
         PropertyName("position"),
         PropertyName("color"),
         PropertyName("radius"),
         PropertyName("text"),
+        PropertyName("elide"),
+        PropertyName("border.color"),
+        PropertyName("border.width"),
+        PropertyName("anchors.verticalCenter"),
+        PropertyName("anchors.left"),
+        PropertyName("anchors.right"),
+        PropertyName("anchors.top"),
+        PropertyName("anchors.bottom"),
+        PropertyName("anchors.fill"),
+        PropertyName("anchors.margins"),
+        PropertyName("font.letterSpacing"),
+        PropertyName("font.pixelSize"),
+        PropertyName("horizontalAlignment"),
+        PropertyName("verticalAlignment"),
+        PropertyName("source"),
+        PropertyName("lineHeight"),
+        PropertyName("lineHeightMode"),
+        PropertyName("wrapMode"),
         PropertyName(),
         PropertyName("states"),
+        PropertyName("to"),
+        PropertyName("from"),
         PropertyName("transitions")
     };
 
@@ -383,7 +396,7 @@ void ModelToTextMerger::dumpRewriteActions(const QString &msg)
     if (DebugRewriteActions) {
         qDebug() << "---->" << qPrintable(msg);
 
-        foreach (RewriteAction *action, m_rewriteActions) {
+        for (RewriteAction *action : std::as_const(m_rewriteActions)) {
             qDebug() << "-----" << qPrintable(action->info());
         }
 
