@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "qtprojectimporter.h"
 
@@ -38,7 +16,6 @@
 #include <utils/qtcassert.h>
 #include <utils/temporarydirectory.h>
 
-#include <QFileInfo>
 #include <QList>
 
 using namespace ProjectExplorer;
@@ -57,7 +34,7 @@ QtProjectImporter::QtVersionData
 QtProjectImporter::findOrCreateQtVersion(const Utils::FilePath &qmakePath) const
 {
     QtVersionData result;
-    result.qt = QtVersionManager::version(Utils::equal(&BaseQtVersion::qmakeFilePath, qmakePath));
+    result.qt = QtVersionManager::version(Utils::equal(&QtVersion::qmakeFilePath, qmakePath));
     if (result.qt) {
         // Check if version is a temporary qt
         const int qtId = result.qt->uniqueId();
@@ -90,10 +67,11 @@ Kit *QtProjectImporter::createTemporaryKit(const QtVersionData &versionData,
         }
 
         additionalSetup(k);
+        k->fix();
     });
 }
 
-static BaseQtVersion *versionFromVariant(const QVariant &v)
+static QtVersion *versionFromVariant(const QVariant &v)
 {
     bool ok;
     const int qtId = v.toInt(&ok);
@@ -106,7 +84,7 @@ void QtProjectImporter::cleanupTemporaryQt(Kit *k, const QVariantList &vl)
     if (vl.isEmpty())
         return; // No temporary Qt
     QTC_ASSERT(vl.count() == 1, return);
-    BaseQtVersion *version = versionFromVariant(vl.at(0));
+    QtVersion *version = versionFromVariant(vl.at(0));
     QTC_ASSERT(version, return);
     QtVersionManager::removeVersion(version);
     QtKitAspect::setQtVersion(k, nullptr); // Always mark Kit as not using this Qt
@@ -118,8 +96,8 @@ void QtProjectImporter::persistTemporaryQt(Kit *k, const QVariantList &vl)
         return; // No temporary Qt
     QTC_ASSERT(vl.count() == 1, return);
     const QVariant data = vl.at(0);
-    BaseQtVersion *tmpVersion = versionFromVariant(data);
-    BaseQtVersion *actualVersion = QtKitAspect::qtVersion(k);
+    QtVersion *tmpVersion = versionFromVariant(data);
+    QtVersion *actualVersion = QtKitAspect::qtVersion(k);
 
     // User changed Kit away from temporary Qt that was set up:
     if (tmpVersion && actualVersion != tmpVersion)
@@ -174,7 +152,7 @@ public:
         m_testData(testData)
     { }
 
-    QStringList importCandidates() override;
+    FilePaths importCandidates() override { return {}; }
 
     bool allDeleted() const { return m_deletedTestData.count() == m_testData.count();}
 
@@ -194,11 +172,6 @@ private:
     QList<Kit *> m_deletedKits;
 };
 
-QStringList TestQtProjectImporter::importCandidates()
-{
-    return QStringList();
-}
-
 QList<void *> TestQtProjectImporter::examineDirectory(const Utils::FilePath &importPath,
                                                       QString *warningMessage) const
 {
@@ -232,7 +205,7 @@ Kit *TestQtProjectImporter::createKit(void *directoryData) const
     // New temporary kit:
     return createTemporaryKit(findOrCreateQtVersion(dd->qmakePath),
                               [dd](Kit *k) {
-        BaseQtVersion *qt = QtKitAspect::qtVersion(k);
+        QtVersion *qt = QtKitAspect::qtVersion(k);
         QMap<Utils::Id, QVariant> toKeep;
         for (const Utils::Id &key : k->allKeys()) {
             if (key.toString().startsWith("PE.tmp."))
@@ -271,22 +244,25 @@ void TestQtProjectImporter::deleteDirectoryData(void *directoryData) const
     delete static_cast<DirectoryData *>(directoryData);
 }
 
-static QStringList additionalFilesToCopy(const BaseQtVersion *qt)
+static QStringList additionalFilesToCopy(const QtVersion *qt)
 {
     // This is a hack and only works with local, "standard" installations of Qt
-    const int major = qt->qtVersion().majorVersion;
+    const int major = qt->qtVersion().majorVersion();
     if (major >= 6) {
         if (HostOsInfo::isMacHost()) {
             return {"lib/QtCore.framework/Versions/A/QtCore"};
         } else if (HostOsInfo::isWindowsHost()) {
             const QString release = QString("bin/Qt%1Core.dll").arg(major);
             const QString debug = QString("bin/Qt%1Cored.dll").arg(major);
+            const QString mingwGcc("bin/libgcc_s_seh-1.dll");
+            const QString mingwStd("bin/libstdc++-6.dll");
+            const QString mingwPthread("bin/libwinpthread-1.dll");
             const FilePath base = qt->qmakeFilePath().parentDir().parentDir();
-            if (base.pathAppended(release).exists())
-                return {release};
-            if (base.pathAppended(debug).exists())
-                return {debug};
-            return {release};
+            const QStringList allFiles = {release, debug, mingwGcc, mingwStd, mingwPthread};
+            const QStringList existingFiles = Utils::filtered(allFiles, [&base](const QString &f) {
+                return base.pathAppended(f).exists();
+            });
+            return !existingFiles.empty() ? existingFiles : QStringList(release);
         } else if (HostOsInfo::isLinuxHost()) {
             const QString core = QString("lib/libQt%1Core.so.%1").arg(major);
             const QDir base(qt->qmakeFilePath().parentDir().parentDir().pathAppended("lib").toString());
@@ -297,7 +273,7 @@ static QStringList additionalFilesToCopy(const BaseQtVersion *qt)
     return {};
 }
 
-static Utils::FilePath setupQmake(const BaseQtVersion *qt, const QString &path)
+static Utils::FilePath setupQmake(const QtVersion *qt, const QString &path)
 {
     // This is a hack and only works with local, "standard" installations of Qt
     const FilePath qmake = qt->qmakeFilePath().canonicalPath();
@@ -402,7 +378,7 @@ void QtSupportPlugin::testQtProjectImporter_oneProject()
     Kit *defaultKit = KitManager::defaultKit();
     QVERIFY(defaultKit);
 
-    BaseQtVersion *defaultQt = QtKitAspect::qtVersion(defaultKit);
+    QtVersion *defaultQt = QtKitAspect::qtVersion(defaultKit);
     QVERIFY(defaultQt);
 
     const Utils::TemporaryDirectory tempDir1("tmp1");
@@ -415,15 +391,15 @@ void QtSupportPlugin::testQtProjectImporter_oneProject()
     // Customize kit numbers 1 and 2:
     QtKitAspect::setQtVersion(kitTemplates[1], nullptr);
     QtKitAspect::setQtVersion(kitTemplates[2], nullptr);
-    SysRootKitAspect::setSysRoot(kitTemplates[1], Utils::FilePath::fromString("/some/path"));
-    SysRootKitAspect::setSysRoot(kitTemplates[2], Utils::FilePath::fromString("/some/other/path"));
+    SysRootKitAspect::setSysRoot(kitTemplates[1], "/some/path");
+    SysRootKitAspect::setSysRoot(kitTemplates[2], "/some/other/path");
 
     QVector<Utils::FilePath> qmakePaths = {defaultQt->qmakeFilePath(),
                                            setupQmake(defaultQt, tempDir1.path().path()),
                                            setupQmake(defaultQt, tempDir2.path().path())};
 
     for (int i = 1; i < qmakePaths.count(); ++i)
-        QVERIFY(!QtVersionManager::version(Utils::equal(&BaseQtVersion::qmakeFilePath, qmakePaths.at(i))));
+        QVERIFY(!QtVersionManager::version(Utils::equal(&QtVersion::qmakeFilePath, qmakePaths.at(i))));
 
     QList<DirectoryData *> testData;
 
@@ -492,7 +468,7 @@ void QtSupportPlugin::testQtProjectImporter_oneProject()
             QCOMPARE(newQtId, defaultQt->uniqueId());
 
         // VALIDATE: Qt is known to QtVersionManager
-        BaseQtVersion *newQt = QtVersionManager::version(newQtId);
+        QtVersion *newQt = QtVersionManager::version(newQtId);
         QVERIFY(newQt);
 
         // VALIDATE: Qt has the expected qmakePath
@@ -637,7 +613,7 @@ void QtSupportPlugin::testQtProjectImporter_oneProject()
 
     qDeleteAll(testData);
 
-    foreach (Kit *k, toUnregisterLater)
+    for (Kit *k : std::as_const(toUnregisterLater))
         KitManager::deregisterKit(k);
 
     // Delete kit templates:
