@@ -1,44 +1,26 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "basefilefind.h"
+
+#include "refactoringchanges.h"
 #include "textdocument.h"
+#include "texteditortr.h"
 
 #include <aggregation/aggregate.h>
-#include <coreplugin/icore.h>
-#include <coreplugin/progressmanager/progressmanager.h>
-#include <coreplugin/progressmanager/futureprogress.h>
+
 #include <coreplugin/dialogs/readonlyfilesdialog.h>
 #include <coreplugin/documentmanager.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/find/ifindsupport.h>
-#include <texteditor/texteditor.h>
-#include <texteditor/refactoringchanges.h>
+#include <coreplugin/icore.h>
+#include <coreplugin/progressmanager/futureprogress.h>
+#include <coreplugin/progressmanager/progressmanager.h>
+
 #include <utils/algorithm.h>
 #include <utils/fadingindicator.h>
 #include <utils/filesearch.h>
+#include <utils/futuresynchronizer.h>
 #include <utils/qtcassert.h>
 #include <utils/stylehelper.h>
 
@@ -58,25 +40,23 @@ using namespace Core;
 namespace TextEditor {
 namespace Internal {
 
-namespace {
 class InternalEngine : public TextEditor::SearchEngine
 {
 public:
     InternalEngine() : m_widget(new QWidget) {}
     ~InternalEngine() override { delete m_widget;}
-    QString title() const override { return TextEditor::SearchEngine::tr("Internal"); }
-    QString toolTip() const override { return QString(); }
+    QString title() const override { return Tr::tr("Internal"); }
+    QString toolTip() const override { return {}; }
     QWidget *widget() const override { return m_widget; }
-    QVariant parameters() const override { return QVariant(); }
+    QVariant parameters() const override { return {}; }
     void readSettings(QSettings * /*settings*/) override {}
     void writeSettings(QSettings * /*settings*/) const override {}
     QFuture<Utils::FileSearchResultList> executeSearch(
             const TextEditor::FileFindParameters &parameters,
             BaseFileFind *baseFileFind) override
     {
-        auto func = parameters.flags & FindRegularExpression
-                ? Utils::findInFilesRegExp
-                : Utils::findInFiles;
+        const auto func = parameters.flags & FindRegularExpression ? Utils::findInFilesRegExp
+                                                                   : Utils::findInFiles;
 
         return func(parameters.text,
                     baseFileFind->files(parameters.nameFilters, parameters.exclusionFilters,
@@ -94,7 +74,6 @@ public:
 private:
     QWidget *m_widget;
 };
-} // namespace
 
 class SearchEnginePrivate
 {
@@ -112,10 +91,12 @@ public:
 class BaseFileFindPrivate
 {
 public:
-    ~BaseFileFindPrivate() { delete m_internalSearchEngine; }
+    BaseFileFindPrivate() { m_futureSynchronizer.setCancelOnWait(true); }
+
     QPointer<IFindSupport> m_currentFindSupport;
 
-    QLabel *m_resultLabel = 0;
+    Utils::FutureSynchronizer m_futureSynchronizer;
+    QLabel *m_resultLabel = nullptr;
     // models in native path format
     QStringListModel m_filterStrings;
     QStringListModel m_exclusionStrings;
@@ -125,7 +106,7 @@ public:
     QPointer<QComboBox> m_filterCombo;
     QPointer<QComboBox> m_exclusionCombo;
     QVector<SearchEngine *> m_searchEngines;
-    SearchEngine *m_internalSearchEngine;
+    InternalEngine m_internalSearchEngine;
     int m_currentSearchEngineIndex = -1;
 };
 
@@ -182,8 +163,7 @@ void SearchEngine::setEnabled(bool enabled)
 
 BaseFileFind::BaseFileFind() : d(new BaseFileFindPrivate)
 {
-    d->m_internalSearchEngine = new InternalEngine;
-    addSearchEngine(d->m_internalSearchEngine);
+    addSearchEngine(&d->m_internalSearchEngine);
 }
 
 BaseFileFind::~BaseFileFind()
@@ -200,14 +180,14 @@ QStringList BaseFileFind::fileNameFilters() const
 {
     if (d->m_filterCombo)
         return splitFilterUiText(d->m_filterCombo->currentText());
-    return QStringList();
+    return {};
 }
 
 QStringList BaseFileFind::fileExclusionFilters() const
 {
     if (d->m_exclusionCombo)
         return splitFilterUiText(d->m_exclusionCombo->currentText());
-    return QStringList();
+    return {};
 }
 
 SearchEngine *BaseFileFind::currentSearchEngine() const
@@ -230,21 +210,29 @@ void BaseFileFind::setCurrentSearchEngine(int index)
     emit currentSearchEngineChanged();
 }
 
+static QString displayText(const QString &line)
+{
+    QString result = line;
+    auto end = result.end();
+    for (auto it = result.begin(); it != end; ++it) {
+        if (!it->isSpace() && !it->isPrint())
+            *it = QChar('?');
+    }
+    return result;
+}
+
 static void displayResult(QFutureWatcher<FileSearchResultList> *watcher,
                           SearchResult *search, int index)
 {
-    FileSearchResultList results = watcher->resultAt(index);
+    const FileSearchResultList results = watcher->resultAt(index);
     QList<SearchResultItem> items;
-    foreach (const FileSearchResult &result, results) {
+    for (const FileSearchResult &result : results) {
         SearchResultItem item;
-        item.path = QStringList() << QDir::toNativeSeparators(result.fileName);
-        item.mainRange.begin.line = result.lineNumber;
-        item.mainRange.begin.column = result.matchStart;
-        item.mainRange.end = item.mainRange.begin;
-        item.mainRange.end.column += result.matchLength;
-        item.text = result.matchingLine;
-        item.useTextEditorFont = true;
-        item.userData = result.regexpCapturedTexts;
+        item.setFilePath(result.fileName);
+        item.setMainRange(result.lineNumber, result.matchStart, result.matchLength);
+        item.setLineText(displayText(result.matchingLine));
+        item.setUseTextEditorFont(true);
+        item.setUserData(result.regexpCapturedTexts);
         items << item;
     }
     search->addResults(items, SearchResult::AddOrdered);
@@ -253,12 +241,12 @@ static void displayResult(QFutureWatcher<FileSearchResultList> *watcher,
 void BaseFileFind::runNewSearch(const QString &txt, FindFlags findFlags,
                                     SearchResultWindow::SearchMode searchMode)
 {
-    d->m_currentFindSupport = 0;
+    d->m_currentFindSupport = nullptr;
     if (d->m_filterCombo)
         updateComboEntries(d->m_filterCombo, true);
     if (d->m_exclusionCombo)
         updateComboEntries(d->m_exclusionCombo, true);
-    QString tooltip = toolTip();
+    const QString tooltip = toolTip();
 
     SearchResult *search = SearchResultWindow::instance()->startNewSearch(
                 label(),
@@ -275,31 +263,33 @@ void BaseFileFind::runNewSearch(const QString &txt, FindFlags findFlags,
     parameters.additionalParameters = additionalParameters();
     parameters.searchEngineParameters = currentSearchEngine()->parameters();
     parameters.searchEngineIndex = d->m_currentSearchEngineIndex;
-    search->setUserData(qVariantFromValue(parameters));
-    connect(search, &SearchResult::activated, this, &BaseFileFind::openEditor);
+    search->setUserData(QVariant::fromValue(parameters));
+    connect(search, &SearchResult::activated, this, [this, search](const SearchResultItem &item) {
+        openEditor(search, item);
+    });
     if (searchMode == SearchResultWindow::SearchAndReplace)
         connect(search, &SearchResult::replaceButtonClicked, this, &BaseFileFind::doReplace);
     connect(search, &SearchResult::visibilityChanged, this, &BaseFileFind::hideHighlightAll);
-    connect(search, &SearchResult::searchAgainRequested, this, &BaseFileFind::searchAgain);
+    connect(search, &SearchResult::searchAgainRequested, this, [this, search] {
+        searchAgain(search);
+    });
     connect(this, &BaseFileFind::enabledChanged, search, &SearchResult::requestEnabledCheck);
-    connect(search, &SearchResult::requestEnabledCheck, this, &BaseFileFind::recheckEnabled);
+    connect(search, &SearchResult::requestEnabledCheck, this, [this, search] {
+        recheckEnabled(search);
+    });
 
     runSearch(search);
 }
 
 void BaseFileFind::runSearch(SearchResult *search)
 {
-    FileFindParameters parameters = search->userData().value<FileFindParameters>();
-    CountingLabel *label = new CountingLabel;
-    connect(search, &SearchResult::countChanged, label, &CountingLabel::updateCount);
-    CountingLabel *statusLabel = new CountingLabel;
-    connect(search, &SearchResult::countChanged, statusLabel, &CountingLabel::updateCount);
+    const FileFindParameters parameters = search->userData().value<FileFindParameters>();
     SearchResultWindow::instance()->popup(IOutputPane::Flags(IOutputPane::ModeSwitch|IOutputPane::WithFocus));
-    QFutureWatcher<FileSearchResultList> *watcher = new QFutureWatcher<FileSearchResultList>();
+    auto watcher = new QFutureWatcher<FileSearchResultList>();
     watcher->setPendingResultsLimit(1);
     // search is deleted if it is removed from search panel
     connect(search, &QObject::destroyed, watcher, &QFutureWatcherBase::cancel);
-    connect(search, &SearchResult::cancelled, watcher, &QFutureWatcherBase::cancel);
+    connect(search, &SearchResult::canceled, watcher, &QFutureWatcherBase::cancel);
     connect(search, &SearchResult::paused, watcher, [watcher](bool paused) {
         if (!paused || watcher->isRunning()) // guard against pausing when the search is finished
             watcher->setPaused(paused);
@@ -312,11 +302,16 @@ void BaseFileFind::runSearch(SearchResult *search)
     connect(watcher, &QFutureWatcherBase::finished, search, [watcher, search]() {
         search->finishSearch(watcher->isCanceled());
     });
-    watcher->setFuture(executeSearch(parameters));
-    FutureProgress *progress =
-        ProgressManager::addTask(watcher->future(), tr("Searching"), Constants::TASK_SEARCH);
-    progress->setWidget(label);
-    progress->setStatusBarWidget(statusLabel);
+    QFuture<FileSearchResultList> future = executeSearch(parameters);
+    watcher->setFuture(future);
+    d->m_futureSynchronizer.addFuture(future);
+    FutureProgress *progress = ProgressManager::addTask(future,
+                                                        Tr::tr("Searching"),
+                                                        Constants::TASK_SEARCH);
+    connect(search, &SearchResult::countChanged, progress, [progress](int c) {
+        progress->setSubtitle(Tr::tr("%n found.", nullptr, c));
+    });
+    progress->setSubtitleVisibleInStatusBar(true);
     connect(progress, &FutureProgress::clicked, search, &SearchResult::popup);
 }
 
@@ -341,10 +336,10 @@ void BaseFileFind::doReplace(const QString &text,
                              const QList<SearchResultItem> &items,
                              bool preserveCase)
 {
-    QStringList files = replaceAll(text, items, preserveCase);
+    const FilePaths files = replaceAll(text, items, preserveCase);
     if (!files.isEmpty()) {
-        Utils::FadingIndicator::showText(ICore::mainWindow(),
-            tr("%n occurrences replaced.", 0, items.size()),
+        Utils::FadingIndicator::showText(ICore::dialogParent(),
+            Tr::tr("%n occurrences replaced.", nullptr, items.size()),
             Utils::FadingIndicator::SmallText);
         DocumentManager::notifyFilesChangedInternally(files);
         SearchResultWindow::instance()->hide();
@@ -385,13 +380,12 @@ QList<QPair<QWidget *, QWidget *>> BaseFileFind::createPatternWidgets()
     d->m_exclusionCombo->setToolTip(msgFilePatternToolTip());
     exclusionLabel->setBuddy(d->m_exclusionCombo);
     syncComboWithSettings(d->m_exclusionCombo, d->m_exclusionSetting);
-    return { qMakePair(filterLabel,    d->m_filterCombo),
-             qMakePair(exclusionLabel, d->m_exclusionCombo) };
+    return {{filterLabel, d->m_filterCombo}, {exclusionLabel, d->m_exclusionCombo}};
 }
 
 void BaseFileFind::writeCommonSettings(QSettings *settings)
 {
-    std::function<QStringList(const QStringList &)> fromNativeSeparators = [](const QStringList &files) {
+    const auto fromNativeSeparators = [](const QStringList &files) -> QStringList {
         return Utils::transform(files, &QDir::fromNativeSeparators);
     };
 
@@ -404,7 +398,7 @@ void BaseFileFind::writeCommonSettings(QSettings *settings)
         settings->setValue("currentExclusionFilter",
                            QDir::fromNativeSeparators(d->m_exclusionCombo->currentText()));
 
-    foreach (SearchEngine *searchEngine, d->m_searchEngines)
+    for (const SearchEngine *searchEngine : std::as_const(d->m_searchEngines))
         searchEngine->writeSettings(settings);
     settings->setValue("currentSearchEngineIndex", d->m_currentSearchEngineIndex);
 }
@@ -412,13 +406,13 @@ void BaseFileFind::writeCommonSettings(QSettings *settings)
 void BaseFileFind::readCommonSettings(QSettings *settings, const QString &defaultFilter,
                                       const QString &defaultExclusionFilter)
 {
-    std::function<QStringList(const QStringList &)> toNativeSeparators = [](const QStringList &files) {
+    const auto toNativeSeparators = [](const QStringList &files) -> QStringList {
         return Utils::transform(files, &QDir::toNativeSeparators);
     };
 
-    QStringList filters = settings->value("filters").toStringList();
-    if (filters.isEmpty())
-        filters << defaultFilter;
+    const QStringList filterSetting = settings->value("filters").toStringList();
+    const QStringList filters = filterSetting.isEmpty() ? QStringList(defaultFilter)
+                                                        : filterSetting;
     const QVariant currentFilter = settings->value("currentFilter");
     d->m_filterSetting = currentFilter.isValid() ? currentFilter.toString()
                                                  : filters.first();
@@ -427,7 +421,7 @@ void BaseFileFind::readCommonSettings(QSettings *settings, const QString &defaul
         syncComboWithSettings(d->m_filterCombo, d->m_filterSetting);
 
     QStringList exclusionFilters = settings->value("exclusionFilters").toStringList();
-    if (exclusionFilters.isEmpty())
+    if (!exclusionFilters.contains(defaultExclusionFilter))
         exclusionFilters << defaultExclusionFilter;
     const QVariant currentExclusionFilter = settings->value("currentExclusionFilter");
     d->m_exclusionSetting = currentExclusionFilter.isValid() ? currentExclusionFilter.toString()
@@ -436,27 +430,26 @@ void BaseFileFind::readCommonSettings(QSettings *settings, const QString &defaul
     if (d->m_exclusionCombo)
         syncComboWithSettings(d->m_exclusionCombo, d->m_exclusionSetting);
 
-    foreach (SearchEngine* searchEngine, d->m_searchEngines)
+    for (SearchEngine* searchEngine : std::as_const(d->m_searchEngines))
         searchEngine->readSettings(settings);
     const int currentSearchEngineIndex = settings->value("currentSearchEngineIndex", 0).toInt();
     syncSearchEngineCombo(currentSearchEngineIndex);
 }
 
-void BaseFileFind::openEditor(const SearchResultItem &item)
+void BaseFileFind::openEditor(SearchResult *result, const SearchResultItem &item)
 {
-    SearchResult *result = qobject_cast<SearchResult *>(sender());
-    FileFindParameters parameters = result->userData().value<FileFindParameters>();
+    const FileFindParameters parameters = result->userData().value<FileFindParameters>();
     IEditor *openedEditor =
             d->m_searchEngines[parameters.searchEngineIndex]->openEditor(item, parameters);
     if (!openedEditor)
-        EditorManager::openEditorAtSearchResult(item, EditorManager::DoNotSwitchToDesignMode);
+        EditorManager::openEditorAtSearchResult(item, Id(), EditorManager::DoNotSwitchToDesignMode);
     if (d->m_currentFindSupport)
         d->m_currentFindSupport->clearHighlights();
-    d->m_currentFindSupport = 0;
+    d->m_currentFindSupport = nullptr;
     if (!openedEditor)
         return;
     // highlight results
-    if (IFindSupport *findSupport = Aggregation::query<IFindSupport>(openedEditor->widget())) {
+    if (auto findSupport = Aggregation::query<IFindSupport>(openedEditor->widget())) {
         d->m_currentFindSupport = findSupport;
         d->m_currentFindSupport->highlightAll(parameters.text, parameters.flags);
     }
@@ -468,83 +461,76 @@ void BaseFileFind::hideHighlightAll(bool visible)
         d->m_currentFindSupport->clearHighlights();
 }
 
-void BaseFileFind::searchAgain()
+void BaseFileFind::searchAgain(SearchResult *search)
 {
-    SearchResult *search = qobject_cast<SearchResult *>(sender());
     search->restart();
     runSearch(search);
 }
 
-void BaseFileFind::recheckEnabled()
+void BaseFileFind::recheckEnabled(SearchResult *search)
 {
-    SearchResult *search = qobject_cast<SearchResult *>(sender());
     if (!search)
         return;
     search->setSearchAgainEnabled(isEnabled());
 }
 
-QStringList BaseFileFind::replaceAll(const QString &text,
-                                     const QList<SearchResultItem> &items,
-                                     bool preserveCase)
+FilePaths BaseFileFind::replaceAll(const QString &text,
+                                   const QList<SearchResultItem> &items,
+                                   bool preserveCase)
 {
     if (items.isEmpty())
-        return QStringList();
+        return {};
 
     RefactoringChanges refactoring;
 
-    QHash<QString, QList<SearchResultItem> > changes;
-    foreach (const SearchResultItem &item, items)
-        changes[QDir::fromNativeSeparators(item.path.first())].append(item);
+    QHash<FilePath, QList<SearchResultItem> > changes;
+    for (const SearchResultItem &item : items)
+        changes[FilePath::fromUserInput(item.path().constFirst())].append(item);
 
     // Checking for files without write permissions
-    QHashIterator<QString, QList<SearchResultItem> > it(changes);
-    QSet<QString> roFiles;
-    while (it.hasNext()) {
-        it.next();
-        const QFileInfo fileInfo(it.key());
-        if (!fileInfo.isWritable())
+    QSet<FilePath> roFiles;
+    for (auto it = changes.cbegin(), end = changes.cend(); it != end; ++it) {
+        if (!it.key().isWritableFile())
             roFiles.insert(it.key());
     }
 
     // Query the user for permissions
     if (!roFiles.isEmpty()) {
-        ReadOnlyFilesDialog roDialog(roFiles.toList(), ICore::mainWindow());
-        roDialog.setShowFailWarning(true, tr("Aborting replace."));
+        ReadOnlyFilesDialog roDialog(Utils::toList(roFiles), ICore::dialogParent());
+        roDialog.setShowFailWarning(true, Tr::tr("Aborting replace."));
         if (roDialog.exec() == ReadOnlyFilesDialog::RO_Cancel)
-            return QStringList();
+            return {};
     }
 
-    it.toFront();
-    while (it.hasNext()) {
-        it.next();
-        const QString fileName = it.key();
+    for (auto it = changes.cbegin(), end = changes.cend(); it != end; ++it) {
+        const FilePath filePath = it.key();
         const QList<SearchResultItem> changeItems = it.value();
 
         ChangeSet changeSet;
-        RefactoringFilePtr file = refactoring.file(fileName);
-        QSet<QPair<int, int> > processed;
-        foreach (const SearchResultItem &item, changeItems) {
-            const QPair<int, int> &p = qMakePair(item.mainRange.begin.line,
-                                                 item.mainRange.begin.column);
+        RefactoringFilePtr file = refactoring.file(filePath);
+        QSet<QPair<int, int>> processed;
+        for (const SearchResultItem &item : changeItems) {
+            const QPair<int, int> p{item.mainRange().begin.line, item.mainRange().begin.column};
             if (processed.contains(p))
                 continue;
             processed.insert(p);
 
             QString replacement;
-            if (item.userData.canConvert<QStringList>() && !item.userData.toStringList().isEmpty()) {
-                replacement = Utils::expandRegExpReplacement(text, item.userData.toStringList());
+            if (item.userData().canConvert<QStringList>() && !item.userData().toStringList().isEmpty()) {
+                replacement = Utils::expandRegExpReplacement(text, item.userData().toStringList());
             } else if (preserveCase) {
-                const QString originalText = (item.mainRange.length() == 0) ? item.text
-                                                                            : item.mainRange.mid(text);
+                const QString originalText = (item.mainRange().length(item.lineText()) == 0)
+                                                 ? item.lineText()
+                                                 : item.mainRange().mid(item.lineText());
                 replacement = Utils::matchCaseReplacement(originalText, text);
             } else {
                 replacement = text;
             }
 
-            const int start = file->position(item.mainRange.begin.line,
-                                             item.mainRange.begin.column + 1);
-            const int end = file->position(item.mainRange.end.line,
-                                           item.mainRange.end.column + 1);
+            const int start = file->position(item.mainRange().begin.line,
+                                             item.mainRange().begin.column + 1);
+            const int end = file->position(item.mainRange().end.line,
+                                           item.mainRange().end.column + 1);
             changeSet.replace(start, end, replacement);
         }
         file->setChangeSet(changeSet);
@@ -566,23 +552,5 @@ QFuture<FileSearchResultList> BaseFileFind::executeSearch(const FileFindParamete
 
 namespace Internal {
 
-CountingLabel::CountingLabel()
-{
-    setAlignment(Qt::AlignCenter);
-    // ### TODO this setup should be done by style
-    QFont f = font();
-    f.setBold(true);
-    f.setPointSizeF(StyleHelper::sidebarFontSize());
-    setFont(f);
-    setPalette(StyleHelper::sidebarFontPalette(palette()));
-    setProperty("_q_custom_style_disabled", QVariant(true));
-    updateCount(0);
-}
-
-void CountingLabel::updateCount(int count)
-{
-    setText(BaseFileFind::tr("%n found.", nullptr, count));
-}
-
 } // namespace Internal
 } // namespace TextEditor
