@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #pragma once
 
@@ -39,6 +17,9 @@ enum TextStyle : quint8 {
     C_SELECTION,
     C_LINE_NUMBER,
     C_SEARCH_RESULT,
+    C_SEARCH_RESULT_ALT1,
+    C_SEARCH_RESULT_ALT2,
+    C_SEARCH_RESULT_CONTAINING_FUNCTION,
     C_SEARCH_SCOPE,
     C_PARENTHESES,
     C_PARENTHESES_MISMATCH,
@@ -52,7 +33,9 @@ enum TextStyle : quint8 {
     C_NUMBER,
     C_STRING,
     C_TYPE,
+    C_NAMESPACE,
     C_LOCAL,
+    C_PARAMETER,
     C_GLOBAL,
     C_FIELD,
     C_ENUMERATION,
@@ -64,6 +47,7 @@ enum TextStyle : quint8 {
     C_OVERLOADED_OPERATOR,
     C_PUNCTUATION,
     C_PREPROCESSOR,
+    C_MACRO,
     C_LABEL,
     C_COMMENT,
     C_DOXYGEN_COMMENT,
@@ -81,7 +65,6 @@ enum TextStyle : quint8 {
     C_QML_STATE_NAME,
     C_BINDING,
 
-
     C_DISABLED_CODE,
 
     C_ADDED_LINE,
@@ -111,6 +94,18 @@ enum TextStyle : quint8 {
     C_DECLARATION,
     C_FUNCTION_DEFINITION,
     C_OUTPUT_ARGUMENT,
+    C_STATIC_MEMBER,
+
+    C_COCO_CODE_ADDED,
+    C_COCO_PARTIALLY_COVERED,
+    C_COCO_NOT_COVERED,
+    C_COCO_FULLY_COVERED,
+    C_COCO_MANUALLY_VALIDATED,
+    C_COCO_DEAD_CODE,
+    C_COCO_EXECUTION_COUNT_TOO_LOW,
+    C_COCO_NOT_COVERED_INFO,
+    C_COCO_COVERED_INFO,
+    C_COCO_MANUALLY_VALIDATED_INFO,
 
     C_LAST_STYLE_SENTINEL
 };
@@ -124,6 +119,7 @@ const char G_COPYPASTE[]           = "TextEditor.CopyPasteGroup";
 const char G_SELECT[]              = "TextEditor.SelectGroup";
 const char G_BOM[]                 = "TextEditor.BomGroup";
 const char COMPLETE_THIS[]         = "TextEditor.CompleteThis";
+const char FUNCTION_HINT[]         = "TextEditor.FunctionHint";
 const char QUICKFIX_THIS[]         = "TextEditor.QuickFix";
 const char SHOWCONTEXTMENU[]       = "TextEditor.ShowContextMenu";
 const char CREATE_SCRATCH_BUFFER[] = "TextEditor.CreateScratchBuffer";
@@ -154,6 +150,7 @@ const char MOVE_LINE_UP[]          = "TextEditor.MoveLineUp";
 const char MOVE_LINE_DOWN[]        = "TextEditor.MoveLineDown";
 const char COPY_LINE_UP[]          = "TextEditor.CopyLineUp";
 const char COPY_LINE_DOWN[]        = "TextEditor.CopyLineDown";
+const char COPY_WITH_HTML[]        = "TextEditor.CopyWithHtml";
 const char JOIN_LINES[]            = "TextEditor.JoinLines";
 const char INSERT_LINE_ABOVE[]     = "TextEditor.InsertLineAboveCurrentLine";
 const char INSERT_LINE_BELOW[]     = "TextEditor.InsertLineBelowCurrentLine";
@@ -162,6 +159,8 @@ const char LOWERCASE_SELECTION[]   = "TextEditor.LowercaseSelection";
 const char SORT_SELECTED_LINES[]   = "TextEditor.SortSelectedLines";
 const char CUT_LINE[]              = "TextEditor.CutLine";
 const char COPY_LINE[]             = "TextEditor.CopyLine";
+const char ADD_SELECT_NEXT_FIND_MATCH[] = "TextEditor.AddSelectionNextFindMatch";
+const char ADD_CURSORS_TO_LINE_ENDS[] = "TextEditor.AddCursorsAtLineEnd";
 const char DUPLICATE_SELECTION[]   = "TextEditor.DuplicateSelection";
 const char DUPLICATE_SELECTION_AND_COMMENT[] = "TextEditor.DuplicateSelectionAndComment";
 const char DELETE_LINE[]           = "TextEditor.DeleteLine";
@@ -200,6 +199,7 @@ const char INFO_MISSING_SYNTAX_DEFINITION[] = "TextEditor.InfoSyntaxDefinition";
 const char INFO_MULTIPLE_SYNTAX_DEFINITIONS[] = "TextEditor.InfoMultipleSyntaxDefinitions";
 const char TASK_OPEN_FILE[]        = "TextEditor.Task.OpenFile";
 const char CIRCULAR_PASTE[]        = "TextEditor.CircularPaste";
+const char NO_FORMAT_PASTE[]       = "TextEditor.NoFormatPaste";
 const char SWITCH_UTF8BOM[]        = "TextEditor.SwitchUtf8bom";
 const char INDENT[]        = "TextEditor.Indent";
 const char UNINDENT[]        = "TextEditor.Unindent";
@@ -231,6 +231,7 @@ const char SNIPPET_EDITOR_ID[]     = "TextEditor.SnippetEditor";
 const char TEXT_SNIPPET_GROUP_ID[] = "Text";
 
 const char GLOBAL_SETTINGS_ID[]    = "Global";
+const char GENERIC_PROPOSAL_ID[] = "TextEditor.GenericProposalId";
 
 /**
  * Delay before tooltip will be shown near completion assistant proposal
