@@ -1,27 +1,5 @@
-############################################################################
-#
 # Copyright (C) 2022 The Qt Company Ltd.
-# Contact: https://www.qt.io/licensing/
-#
-# This file is part of Qt Creator.
-#
-# Commercial License Usage
-# Licensees holding valid commercial Qt licenses may use this file in
-# accordance with the commercial license agreement provided with the
-# Software or, alternatively, in accordance with the terms contained in
-# a written agreement between you and The Qt Company. For licensing terms
-# and conditions see https://www.qt.io/terms-conditions. For further
-# information use the contact form at https://www.qt.io/contact-us.
-#
-# GNU General Public License Usage
-# Alternatively, this file may be used under the terms of the GNU
-# General Public License version 3 as published by the Free Software
-# Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-# included in the packaging of this file. Please review the following
-# information to ensure the GNU General Public License requirements will
-# be met: https://www.gnu.org/licenses/gpl-3.0.html.
-#
-############################################################################
+# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 import tempfile
 
@@ -302,11 +280,12 @@ def addCurrentCreatorDocumentation():
     clickButton(waitForObject("{type='QPushButton' name='addButton' visible='1' text='Add...'}"))
     selectFromFileDialog(docPath)
     try:
-        waitForObject("{type='QMessageBox' unnamed='1' visible='1' "
-                      "text~='Unable to register documentation.*'}", 3000)
+        windowStr = ("{type='QMessageBox' unnamed='1' visible='1' "
+                      "text~='Unable to register documentation.*'}")
+        waitForObject(windowStr, 3000)
         test.passes("Qt Creator's documentation found already registered.")
         clickButton(waitForObject("{type='QPushButton' text='OK' unnamed='1' visible='1' "
-                                  "container={name='groupBox' type='QGroupBox' visible='1'}}"))
+                                  "window=%s}" % windowStr))
     except:
         test.fail("Added Qt Creator's documentation explicitly.")
     clickButton(waitForObject(":Options.OK_QPushButton"))
@@ -363,7 +342,7 @@ def getConfiguredKits():
     # update collected Qt versions with their configured device and version
     iterateKits(False, True, __setQtVersionForKit__, kitsWithQtVersionName)
     # merge defined target names with their configured Qt versions and devices
-    for kit, qtVersion in kitsWithQtVersionName.iteritems():
+    for kit, qtVersion in kitsWithQtVersionName.items():
         if qtVersion in qtVersionNames:
             result.append(kit)
         else:
@@ -534,8 +513,13 @@ def progressBarWait(timeout=60000, warn=True):
     checkIfObjectExists(":Qt Creator_Core::Internal::ProgressBar", False, timeout)
 
 def readFile(filename):
-    with open(filename, "r") as f:
-        return f.read()
+    try:
+        with open(filename, "r") as f:
+            return f.read()
+    except:
+        # Read file as binary
+        with open(filename, "rb") as f:
+            return f.read()
 
 def simpleFileName(navigatorFileName):
     # try to find the last part of the given name, assume it's inside a (folder) structure
